@@ -8,7 +8,12 @@ would appreciate credit if this program or parts of it are used.
 
 */
 
+#include <stdio.h>
 #include "tclInt.h"
+/*#include <varargs.h>		tclInt.h drags in varargs.h.  Since Pyramid */
+/*				objects to including varargs.h twice, just */
+/*				omit this one. */
+#include "string.h"
 #include "Dbg.h"
 
 #ifndef TRUE
@@ -24,9 +29,15 @@ static int zero();
 
 static Dbg_InterProc *interactor = simple_interactor;
 static Dbg_IgnoreFuncsProc *ignoreproc = zero;
+static Dbg_OutputProc *printproc = 0;
+
+static void print();
+
 static int debugger_active = FALSE;
 
-char *debug_varname = "dbg";
+/* this is not externally documented anywhere as of yet */
+char *Dbg_VarName = "dbg";
+
 #define DEFAULT_COMPRESS	0
 static int compress = DEFAULT_COMPRESS;
 #define DEFAULT_WIDTH		75	/* leave a little space for printing */
@@ -44,11 +55,16 @@ static int step_count = 1;	/* count next/step */
 static char viewFrameName[FRAMENAMELEN];/* destination frame name for up/down */
 
 static CallFrame *goalFramePtr;	/* destination for next/return */
+static int goalNumLevel;	/* destination for Next */
 
 static enum debug_cmd {
-	step, next, ret, cont, up, down, where
+	none, step, next, ret, cont, up, down, where, Next
 } debug_cmd;
 
+/* info about last action to use as a default */
+static enum debug_cmd last_action_cmd = next;
+static int last_step_count = 1;
+
 /* this acts as a strobe (while testing breakpoints).  It is set to true */
 /* every time a new debugger command is issued that is an action */
 static debug_new_action;
@@ -89,32 +105,33 @@ breakpoint_new()
 
 static
 void
-breakpoint_print(b)
+breakpoint_print(interp,b)
+Tcl_Interp *interp;
 struct breakpoint *b;
 {
-	printf("breakpoint %d: ",b->id);
+	print(interp,"breakpoint %d: ",b->id);
 
 	if (b->re) {
-		printf("-re \"%s\" ",b->pat);
+		print(interp,"-re \"%s\" ",b->pat);
 	} else if (b->pat) {
-		printf("-glob \"%s\" ",b->pat);
+		print(interp,"-glob \"%s\" ",b->pat);
 	} else if (b->line != NO_LINE) {
 		if (b->file) {
-			printf("%s:",b->file);
+			print(interp,"%s:",b->file);
 		}
-		printf("%d ",b->line);
+		print(interp,"%d ",b->line);
 	}
 
 	if (b->expr)
-		printf("if {%s} ",b->expr);
+		print(interp,"if {%s} ",b->expr);
 
 	if (b->cmd)
-		printf("then {%s}",b->cmd);
+		print(interp,"then {%s}",b->cmd);
 
 	putchar('\n');
 }
 
-void
+static void
 save_re_matches(interp,re)
 Tcl_Interp *interp;
 regexp *re;
@@ -131,7 +148,7 @@ regexp *re;
 		/* temporarily null-terminate in middle */
 		match_char = *re->endp[i];
 		*re->endp[i] = 0;
-		Tcl_SetVar2(interp,debug_varname,name,re->startp[i],0);
+		Tcl_SetVar2(interp,Dbg_VarName,name,re->startp[i],0);
 
 		/* undo temporary null-terminator */
 		*re->endp[i] = match_char;
@@ -146,7 +163,11 @@ char *cmd;		/* command about to be executed */
 struct breakpoint *bp;	/* breakpoint to test */
 {
 	if (bp->re) {
+#if TCL_MAJOR_VERSION == 6
 		if (0 == regexec(bp->re,cmd)) return 0;
+#else
+		if (0 == TclRegExec(bp->re,cmd,cmd)) return 0;
+#endif
 		save_re_matches(interp,bp->re);
 	} else if (bp->pat) {
 		if (0 == Tcl_StringMatch(cmd,bp->pat)) return 0;
@@ -170,7 +191,7 @@ struct breakpoint *bp;	/* breakpoint to test */
 		Tcl_Eval(interp,bp->cmd);
 #endif
 	} else {
-		breakpoint_print(bp);
+		breakpoint_print(interp,bp);
 	}
 
 	return 1;
@@ -273,10 +294,10 @@ char *s;
 	/* worst case is every character takes 4 to printify */
 	need = strlen(s)*4;
 	if (need > destlen) {
-		if (dest && (dest != buf_basic)) free(dest);
-		dest = ckalloc(need+1);
-			destlen = need;
-		}
+		if (dest && (dest != buf_basic)) ckfree(dest);
+		dest = (char *)ckalloc(need+1);
+		destlen = need;
+	}
 
 	for (d = dest;*s;s++) {
 		/* since we check at worst by every 4 bytes, play */
@@ -324,10 +345,10 @@ char *argv[];
 	int arg_index;
 
 	if (buf_width > buf_width_max) {
-		if (buf && (buf != buf_basic)) free(buf);
-		buf = malloc(buf_width + 1);
-			buf_width_max = buf_width;
-		}
+		if (buf && (buf != buf_basic)) ckfree(buf);
+		buf = (char *)ckalloc(buf_width + 1);
+		buf_width_max = buf_width;
+	}
 
 	proc = (0 == strcmp("proc",argv[0]));
 	sprintf(buf,"%.*s",buf_width,argv[0]);
@@ -399,10 +420,11 @@ CallFrame *viewf;	/* view FramePtr */
 	ptr = ((curf == viewf)?'*':' ');
 
 	if (curf == 0) {
-		printf("%c0: %s\n",ptr,print_argv(interp,main_argc,main_argv));
+		print(interp,"%c0: %s\n",
+				ptr,print_argv(interp,main_argc,main_argv));
 	} else {
 		PrintStackBelow(interp,curf->callerVarPtr,viewf);
-		printf("%c%d: %s\n",ptr,curf->level,
+		print(interp,"%c%d: %s\n",ptr,curf->level,
 			print_argv(interp,curf->argc,curf->argv));
 	}
 }
@@ -415,25 +437,58 @@ CallFrame *curf;	/* current FramePtr */
 CallFrame *viewf;	/* view FramePtr */
 int argc;
 char *argv[];
+char *level;
 {
 	PrintStackBelow(interp,curf,viewf);
 	
-	printf(" %d: %s\n",level,print_argv(interp,argc,argv));
+	print(interp," %s: %s\n",level,print_argv(interp,argc,argv));
+}
+
+/* return 0 if goal matches current frame or goal can't be found */
+/*	anywere in frame stack */
+/* else return 1 */
+/* This catches things like a proc called from a Tcl_Eval which in */
+/* turn was not called from a proc but some builtin such as source */
+/* or Tcl_Eval.  These builtin calls to Tcl_Eval lose any knowledge */
+/* the FramePtr from the proc, so we have to search the entire */
+/* stack frame to see if it's still there. */
+static int
+GoalFrame(goal,iptr)
+CallFrame *goal;
+Interp *iptr;
+{
+	CallFrame *cf = iptr->varFramePtr;
+
+	/* if at current level, return success immediately */
+	if (goal == cf) return 0;
+
+	while (cf) {
+		cf = cf->callerVarPtr;
+		if (goal == cf) {
+			/* found, but since it's above us, fail */
+			return 1;
+		}
+	}
+	return 0;
 }
 
 /* debugger's trace handler */
 /*ARGSUSED*/
 static void
 debugger_trap(clientData,interp,level,command,cmdProc,cmdClientData,argc,argv)
-ClientData clientData;
+ClientData clientData;		/* not used */
 Tcl_Interp *interp;
-int level;
+int level;			/* positive number if called by Tcl, -1 if */
+				/* called by Dbg_On in which case we don't */
+				/* know the level */
 char *command;
-int (*cmdProc)();
+int (*cmdProc)();		/* not used */
 ClientData cmdClientData;
 int argc;
 char *argv[];
 {
+	char level_text[6];	/* textual representation of level */
+
 	int break_status;
 	Interp *iPtr = (Interp *)interp;
 
@@ -462,7 +517,10 @@ char *argv[];
 		}
 	}
 
-	if (ignoreproc(interp,argv[0])) return;
+	if ((*ignoreproc)(interp,argv[0])) return;
+
+	/* if level is unknown, use "?" */
+	sprintf(level_text,(level == -1)?"?":"%d",level);
 
 	/* save so we can restore later */
 	trueFramePtr = iPtr->varFramePtr;
@@ -491,10 +549,18 @@ char *argv[];
 		goto start_interp;
 	case next:
 		/* check if we are back at the same level where the next */
-		/* command was issued.  Might be wise to also test */
+		/* command was issued.  Also test */
 		/* against all FramePtrs and if no match, assume that */
 		/* we've missed a return, and so we should break  */
-		if (goalFramePtr != iPtr->varFramePtr) goto finish;
+/*		if (goalFramePtr != iPtr->varFramePtr) goto finish;*/
+		if (GoalFrame(goalFramePtr,iPtr)) goto finish;
+		step_count--;
+		if (step_count > 0) goto finish;
+		goto start_interp;
+	case Next:
+		/* check if we are back at the same level where the next */
+		/* command was issued.  */
+		if (goalNumLevel < iPtr->numLevels) goto finish;
 		step_count--;
 		if (step_count > 0) goto finish;
 		goto start_interp;
@@ -506,7 +572,8 @@ char *argv[];
 
 start_interp:
 	if (print_command_first_time) {
-		printf("%d: %s\n",level,print_argv(interp,1,&command));
+		print(interp,"%s: %s\n",
+				level_text,print_argv(interp,1,&command));
 		print_command_first_time = FALSE;
 	}
 	/* since user is typing a command, don't interrupt it immediately */
@@ -514,7 +581,7 @@ start_interp:
 	debug_suspended = FALSE;
 
 	/* interactor won't return until user gives a debugger cmd */
-	interactor(interp);
+	(*interactor)(interp);
 
 	/* save this so it can be restored after "w" command */
 	viewFramePtr = iPtr->varFramePtr;
@@ -523,7 +590,7 @@ start_interp:
 		/* calculate new frame */
 		if (-1 == TclGetFrame2(interp,trueFramePtr,viewFrameName,
 					&iPtr->varFramePtr,debug_cmd)) {
-			printf("%s\n",interp->result);
+			print(interp,"%s\n",interp->result);
 			Tcl_ResetResult(interp);
 		}
 		goto start_interp;
@@ -542,16 +609,19 @@ start_interp:
 	case next:
 		goalFramePtr = iPtr->varFramePtr;
 		goto finish;
+	case Next:
+		goalNumLevel = iPtr->numLevels;
+		goto finish;
 	case ret:
 		goalFramePtr = iPtr->varFramePtr;
 		if (goalFramePtr == 0) {
-			printf("nowhere to return to\n");
+			print(interp,"nowhere to return to\n");
 			break;
 		}
 		goalFramePtr = goalFramePtr->callerVarPtr;
 		goto finish;
 	case where:
-		PrintStack(interp,iPtr->varFramePtr,viewFramePtr,argc,argv,level);
+		PrintStack(interp,iPtr->varFramePtr,viewFramePtr,argc,argv,level_text);
 		break;
 	}
 
@@ -572,10 +642,12 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-	debug_cmd = (enum debug_cmd)clientData;
 	debug_new_action = TRUE;
+	debug_cmd = *(enum debug_cmd *)clientData;
+	last_action_cmd = debug_cmd;
 
 	step_count = (argc == 1)?1:atoi(argv[1]);
+	last_step_count = step_count;
 	return(TCL_RETURN);
 }
 
@@ -588,7 +660,7 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-	debug_cmd = (enum debug_cmd)clientData;
+	debug_cmd = *(enum debug_cmd *)clientData;
 
 	if (argc == 1) argv[1] = "1";
 	strncpy(viewFrameName,argv[1],FRAMENAMELEN);
@@ -605,8 +677,10 @@ Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-	debug_cmd = (enum debug_cmd)clientData;
 	debug_new_action = TRUE;
+	debug_cmd = *(enum debug_cmd *)clientData;
+	last_action_cmd = debug_cmd;
+
 	return TCL_RETURN;
 }
 
@@ -615,10 +689,10 @@ void
 breakpoint_destroy(b)
 struct breakpoint *b;
 {
-	if (b->file) free(b->file);
-	if (b->pat) free(b->pat);
-	if (b->re) free((char *)b->re);			
-	if (b->cmd) free(b->cmd);
+	if (b->file) ckfree(b->file);
+	if (b->pat) ckfree(b->pat);
+	if (b->re) ckfree((char *)b->re);			
+	if (b->cmd) ckfree(b->cmd);
 
 	/* unlink from chain */
 	if ((b->previous == 0) && (b->next == 0)) {
@@ -633,7 +707,7 @@ struct breakpoint *b;
 		b->next->previous = b->previous;
 	}
 
-	free((char *)b);
+	ckfree((char *)b);
 }
 
 static void
@@ -646,7 +720,8 @@ char *str;
 }
 
 /* return 1 if a string is substring of a flag */
-int flageq(flag,string,minlen)
+static int
+flageq(flag,string,minlen)
 char *flag;
 char *string;
 int minlen;		/* at least this many chars must match */
@@ -681,15 +756,15 @@ char **argv;
 			if (*argv) {
 				buf_width = atoi(*argv);
 				argc--; argv++;
-			} else printf("%d\n",buf_width);
+			} else print(interp,"%d\n",buf_width);
 		} else if (flageq("-compress",*argv,2)) {
 			argc--; argv++;
 			if (*argv) {
 				compress = atoi(*argv);
 				argc--; argv++;
-			} else printf("%d\n",compress);
+			} else print(interp,"%d\n",compress);
 		} else {
-			printf("usage: w [-width #] [-compress 0|1]\n");
+			print(interp,"usage: w [-width #] [-compress 0|1]\n");
 			return TCL_ERROR;
 		}
 	}
@@ -713,7 +788,7 @@ char **argv;
 	argc--; argv++;
 
 	if (argc < 1) {
-		for (b = break_base;b;b=b->next) breakpoint_print(b);
+		for (b = break_base;b;b=b->next) breakpoint_print(interp,b);
 		return(TCL_OK);
 	}
 
@@ -743,7 +818,11 @@ char **argv;
 
 	if (flageq("-regexp",argv[0],2)) {
 		argc--; argv++;
+#if TCL_MAJOR_VERSION == 6
 		if ((argc > 0) && (b->re = regcomp(argv[0]))) {
+#else
+		if ((argc > 0) && (b->re = TclRegComp(argv[0]))) {
+#endif
 			savestr(&b->pat,argv[0]);
 			argc--; argv++;
 		} else {
@@ -776,11 +855,11 @@ char **argv;
 
 		if (TCL_OK == Tcl_GetInt(interp,linep,&b->line)) {
 			argc--; argv++;
-			printf("setting breakpoints by line number is currently unimplemented - use patterns or expressions\n");
+			print(interp,"setting breakpoints by line number is currently unimplemented - use patterns or expressions\n");
 		} else {
 			/* not an int? - unwind & assume it is an expression */
 
-			if (b->file) free(b->file);
+			if (b->file) ckfree(b->file);
 		}
 	}
 
@@ -814,7 +893,7 @@ char **argv;
 		}
 
 		savestr(&b->cmd,argv[0]);
-		}
+	}
 
 	sprintf(interp->result,"%d",b->id);
 	return(TCL_OK);
@@ -828,6 +907,7 @@ char **argv;
 static char *help[] = {
 "s [#]		step into procedure",
 "n [#]		step over procedure",
+"N [#]		step over procedures, commands, and arguments",
 "c		continue",
 "r		continue until return to caller",
 "u [#]		move scope up level",
@@ -860,102 +940,43 @@ char **argv;
 	char **hp;
 
 	for (hp=help;*hp;hp++) {
-		printf("%s\n",*hp);
+		print(interp,"%s\n",*hp);
 	}
 
 	return(TCL_OK);
 }
 
-/* this may seem excessive, but this avoids the explicit test for non-zero */
-/* in the caller, and chances are that that test will always be pointless */
-/*ARGSUSED*/
-static int zero(interp,string)
-Tcl_Interp *interp;
-char *string;
+/* occasionally, we print things larger buf_max but not by much */
+/* see print statements in PrintStack routines for examples */
+#define PAD 80
+
+static void
+print(va_alist)
+va_dcl
 {
-	return 0;
-}
+	Tcl_Interp *interp;
+	char *fmt;
+	va_list args;
 
-static int
-simple_interactor(interp)
-Tcl_Interp *interp;
-{
-	int rc;
-	char *ccmd;		/* pointer to complete command */
-	char line[BUFSIZ];	/* space for partial command */
-	int newcmd = TRUE;
-	Interp *iPtr = (Interp *)interp;
+	va_start(args);
+	interp = va_arg(args,Tcl_Interp *);
+	fmt = va_arg(args,char *);
+	if (!printproc) vprintf(fmt,args);
+	else {
+		static int buf_width_max = DEFAULT_WIDTH+PAD;
+		static char buf_basic[DEFAULT_WIDTH+PAD+1];
+		static char *buf = buf_basic;
 
-#if TCL_MAJOR_VERSION == 6
-	Tcl_CmdBuf buffer;
+		if (buf_width+PAD > buf_width_max) {
+			if (buf && (buf != buf_basic)) ckfree(buf);
+			buf = (char *)ckalloc(buf_width+PAD+1);
+			buf_width_max = buf_width+PAD;
+		}
 
-	if (!(buffer = Tcl_CreateCmdBuf())) {
-		Tcl_AppendElement(interp,"no more space for cmd buffer",0);
-		return(TCL_ERROR);
+		vsprintf(buf,fmt,args);
+		(*printproc)(interp,buf);
 	}
-#else
-	Tcl_DString dstring;
-	Tcl_DStringInit(&dstring);
-#endif
-
-
-	newcmd = TRUE;
-	while (TRUE) {
-		if (newcmd) {
-			printf("dbg%d.%d> ",iPtr->numLevels,iPtr->curEventNum+1);
-		} else {
-			printf("dbg+> ");
-		}
-		fflush(stdout);
-
-		if (0 >= (rc = read(0,line,BUFSIZ))) {
-			if (!newcmd) line[0] = 0;
-			else exit(0);
-		} else line[rc] = '\0';
-
-#if TCL_MAJOR_VERSION == 6
-		if (NULL == (ccmd = Tcl_AssembleCmd(buffer,line))) {
-#else
-		ccmd = Tcl_DStringAppend(&dstring,line,rc);
-		if (!Tcl_CommandComplete(ccmd)) {
-#endif
-			newcmd = FALSE;
-			continue;	/* continue collecting command */
-		}
-		newcmd = TRUE;
-
-		switch (rc = Tcl_RecordAndEval(interp,ccmd,0)) {
-		case TCL_OK:
-			if (*interp->result != 0)
-				printf("%s\n",interp->result);
-			continue;
-		case TCL_ERROR:
-			printf("%s\n",Tcl_GetVar(interp,"errorInfo",TCL_GLOBAL_ONLY));
-			/* since user is typing by hand, we expect lots
-			   of errors, and want to give another chance */
-			continue;
-		case TCL_BREAK:
-		case TCL_CONTINUE:
-#define finish(x)	{rc = x; goto done;}
-			finish(rc);
-		case TCL_RETURN:
-			finish(TCL_OK);
-		default:
-			/* note that ccmd has trailing newline */
-			printf("error %d: %s\n",rc,ccmd);
-			continue;
-		}
-	}
-	/* cannot fall thru here, must jump to label */
- done:
-#if TCL_MAJOR_VERSION == 6
-	/* currently, code guarantees buffer is valid */
-	Tcl_DeleteCmdBuf(buffer);
-#else
-	Tcl_DStringFree(&dstring);
-#endif
-
-	return(rc);
+	va_end(args);
 }
 
 /*ARGSUSED*/
@@ -980,6 +1001,18 @@ Dbg_IgnoreFuncsProc *proc;
 	return tmp;
 }
 
+/*ARGSUSED*/
+Dbg_OutputProc *
+Dbg_Output(interp,proc)
+Tcl_Interp *interp;
+Dbg_OutputProc *proc;
+{
+	Dbg_OutputProc *tmp = printproc;
+	printproc = (proc?proc:0);
+	return tmp;
+}
+
+
 /*ARGSUSED*/
 int
 Dbg_Active(interp)
@@ -996,6 +1029,8 @@ int copy;
 {
 	char **alloc;
 
+	main_argc = argc;
+
 	if (!copy) {
 		main_argv = argv;
 		alloc = 0;
@@ -1009,23 +1044,144 @@ int copy;
 	return alloc;
 }
 
-struct cmd_list {
+static struct cmd_list {
 	char *cmdname;
 	Tcl_CmdProc *cmdproc;
 	enum debug_cmd cmdtype;
-} cmd_list[]  = {{"n", cmdNext,   next},
-		 {"s", cmdNext,   step},
-		 {"c", cmdSimple, cont},
-		 {"r", cmdSimple, ret},
-		 {"w", cmdWhere,  0},
-		 {"b", cmdBreak,  0},
-		 {"u", cmdDir,    up},
-		 {"d", cmdDir,    down},
-		 {"h", cmdHelp,   0},
-		 {0}
+} cmd_list[]  = {
+		{"n", cmdNext,   next},
+		{"s", cmdNext,   step},
+		{"N", cmdNext,   Next},
+		{"c", cmdSimple, cont},
+		{"r", cmdSimple, ret},
+		{"w", cmdWhere,  none},
+		{"b", cmdBreak,  none},
+		{"u", cmdDir,    up},
+		{"d", cmdDir,    down},
+		{"h", cmdHelp,   none},
+		{0}
 };
 
-void
+/* this may seem excessive, but this avoids the explicit test for non-zero */
+/* in the caller, and chances are that that test will always be pointless */
+/*ARGSUSED*/
+static int zero(interp,string)
+Tcl_Interp *interp;
+char *string;
+{
+	return 0;
+}
+
+static int
+simple_interactor(interp)
+Tcl_Interp *interp;
+{
+	int rc;
+	char *ccmd;		/* pointer to complete command */
+	char line[BUFSIZ+1];	/* space for partial command */
+	int newcmd = TRUE;
+	Interp *iPtr = (Interp *)interp;
+
+#if TCL_MAJOR_VERSION == 6
+	Tcl_CmdBuf buffer;
+
+	if (!(buffer = Tcl_CreateCmdBuf())) {
+		Tcl_AppendElement(interp,"no more space for cmd buffer",0);
+		return(TCL_ERROR);
+	}
+#else
+	Tcl_DString dstring;
+	Tcl_DStringInit(&dstring);
+#endif
+
+
+	newcmd = TRUE;
+	while (TRUE) {
+		struct cmd_list *c;
+
+		if (newcmd) {
+			print(interp,"dbg%d.%d> ",iPtr->numLevels,iPtr->curEventNum+1);
+		} else {
+			print(interp,"dbg+> ");
+		}
+		fflush(stdout);
+
+		if (0 >= (rc = read(0,line,BUFSIZ))) {
+			if (!newcmd) line[0] = 0;
+			else exit(0);
+		} else line[rc] = '\0';
+
+#if TCL_MAJOR_VERSION == 6
+		if (NULL == (ccmd = Tcl_AssembleCmd(buffer,line))) {
+#else
+		ccmd = Tcl_DStringAppend(&dstring,line,rc);
+		if (!Tcl_CommandComplete(ccmd)) {
+#endif
+			newcmd = FALSE;
+			continue;	/* continue collecting command */
+		}
+		newcmd = TRUE;
+
+		/* if user pressed return with no cmd, use previous one */
+		if ((ccmd[0] == '\n' || ccmd[0] == '\r') && ccmd[1] == '\0') {
+
+			/* this loop is guaranteed to exit through break */
+			for (c = cmd_list;c->cmdname;c++) {
+				if (c->cmdtype == last_action_cmd) break;
+			}
+
+			/* recreate textual version of command */
+			Tcl_DStringAppend(&dstring,c->cmdname,-1);
+
+			if (c->cmdtype == step ||
+			    c->cmdtype == next ||
+			    c->cmdtype == Next) {
+				char num[10];
+
+				sprintf(num," %d",last_step_count);
+				Tcl_DStringAppend(&dstring,num,-1);
+			}
+		}
+
+		rc = Tcl_RecordAndEval(interp,ccmd,0);
+#if TCL_MAJOR_VERSION != 6
+		Tcl_DStringFree(&dstring);
+#endif
+		switch (rc) {
+		case TCL_OK:
+			if (*interp->result != 0)
+				print(interp,"%s\n",interp->result);
+			continue;
+		case TCL_ERROR:
+			print(interp,"%s\n",Tcl_GetVar(interp,"errorInfo",TCL_GLOBAL_ONLY));
+			/* since user is typing by hand, we expect lots
+			   of errors, and want to give another chance */
+			continue;
+		case TCL_BREAK:
+		case TCL_CONTINUE:
+#define finish(x)	{rc = x; goto done;}
+			finish(rc);
+		case TCL_RETURN:
+			finish(TCL_OK);
+		default:
+			/* note that ccmd has trailing newline */
+			print(interp,"error %d: %s\n",rc,ccmd);
+			continue;
+		}
+	}
+	/* cannot fall thru here, must jump to label */
+ done:
+#if TCL_MAJOR_VERSION == 6
+	/* currently, code guarantees buffer is valid */
+	Tcl_DeleteCmdBuf(buffer);
+#else
+	Tcl_DStringFree(&dstring);
+#endif
+
+	return(rc);
+}
+
+static void
 init_debugger(interp)
 Tcl_Interp *interp;
 {
@@ -1033,14 +1189,14 @@ Tcl_Interp *interp;
 
 	for (c = cmd_list;c->cmdname;c++) {
 		Tcl_CreateCommand(interp,c->cmdname,c->cmdproc,
-			(ClientData)c->cmdtype,(Tcl_CmdDeleteProc *)0);
+			(ClientData)&c->cmdtype,(Tcl_CmdDeleteProc *)0);
 	}
 
 	debug_handle = Tcl_CreateTrace(interp,
 				10000,debugger_trap,(ClientData)0);
 
 	debugger_active = TRUE;
-	Tcl_SetVar2(interp,debug_varname,"active","1",0);
+	Tcl_SetVar2(interp,Dbg_VarName,"active","1",0);
 }
 
 /* allows any other part of the application to jump to the debugger */
@@ -1058,7 +1214,11 @@ int immediate;		/* if true, stop immediately */
 	step_count = 1;
 
 	if (immediate) {
-		interactor(interp);
+		static char *fake_cmd = "--interrupted-- (command_unknown)";
+
+		debugger_trap((ClientData)0,interp,-1,fake_cmd,(int (*)())0,
+					(ClientData)0,1,&fake_cmd);
+/*		(*interactor)(interp);*/
 	}
 }
 
@@ -1076,5 +1236,5 @@ Tcl_Interp *interp;
 
 	Tcl_DeleteTrace(interp,debug_handle);
 	debugger_active = FALSE;
-	Tcl_UnsetVar(interp,debug_varname,TCL_GLOBAL_ONLY);
+	Tcl_UnsetVar(interp,Dbg_VarName,TCL_GLOBAL_ONLY);
 }
