@@ -3,8 +3,6 @@
 #include <stdio.h>
 #include "expect.h"
 
-extern exit();
-
 timedout()
 {
 	fprintf(stderr,"timed out\n");
@@ -68,7 +66,7 @@ main(){
 
 	read_first_move(fd1);
 
-	fd2 = exp_spawnl("chess",0);
+	fd2 = exp_spawnl("chess","chess",(char *)0);
 
 	if (-1 == exp_expectl(fd2,exp_glob,"Chess\r\n",0,exp_end)) exit;
 
