@@ -1,18 +1,37 @@
 /* exp_log.c - logging routines */
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #include <stdio.h>
-#include <varargs.h>
-#include "tcl.h"
-#include "exp_global.h"
+/*#include <varargs.h>		tclInt.h drags in varargs.h.  Since Pyramid */
+/*				objects to including varargs.h twice, just */
+/*				omit this one. */
+#include "tclInt.h"
+#include "expect_comm.h"
+#include "exp_int.h"
 #include "exp_rename.h"
 #include "exp_log.h"
 
+/* uhhhh, these doesn't really belong here, but they've got to go somewhere */
+/* that is shared by both the library and Expect, and this is the only */
+/* convenient place, except for a file of their own */
+int exp_disconnected = FALSE;		/* not disc. from controlling tty */
+void (*exp_close_in_child)() = 0;
+
+void
+exp_close_tcl_files() {
+	int i;
+
+	/* So much for close-on-exec.  Tcl doesn't mark its files that way */
+	/* everything has to be closed explicitly. */
+
+	for (i=3; i<tclNumFiles;i++) close(i);
+}
+
+
 int loguser = TRUE;		/* if TRUE, expect/spawn may write to stdout */
 int logfile_all = FALSE;	/* if TRUE, write log of all interactions */
 				/* despite value of loguser. */
 FILE *logfile = 0;
-FILE *cmdfile = 0;
 FILE *debugfile = 0;
 int exp_is_debugging = FALSE;
 
@@ -32,7 +51,7 @@ int exp_is_debugging = FALSE;
 #define LOGUSER		(loguser || force_stdout)
 /*VARARGS*/
 void
-Log(va_alist)
+exp_log(va_alist)
 va_dcl
 {
 	int force_stdout;
@@ -52,7 +71,7 @@ va_dcl
 /* send to log if open */
 /* use this function for logging the parent/child conversation */
 void
-nflog(buf,force_stdout)
+exp_nflog(buf,force_stdout)
 char *buf;
 int force_stdout;	/* override value of loguser */
 {
@@ -65,7 +84,7 @@ int force_stdout;	/* override value of loguser */
 	if (logfile_all || (LOGUSER && logfile)) {
 		int newlength = exp_copy_out(length);
 		fwrite(exp_out_buffer,1,newlength,logfile);
-}
+	}
 #endif
 }
 #undef LOGUSER
@@ -97,7 +116,7 @@ va_dcl
 /* use this function for error conditions */
 /*VARARGS*/
 void
-errorlog(va_alist)
+exp_errorlog(va_alist)
 va_dcl
 {
 	char *fmt;
@@ -116,10 +135,10 @@ va_dcl
 /* use this function for logging the parent/child conversation */
 /*ARGSUSED*/
 void
-nferrorlog(buf,force_stdout)
+exp_nferrorlog(buf,force_stdout)
 char *buf;
 int force_stdout;	/* not used, only declared here for compat with */
-			/* nflog() */
+			/* exp_nflog() */
 {
 	int length = strlen(buf);
 	fwrite(buf,1,length,stderr);
@@ -138,7 +157,7 @@ static char *outp;	/* pointer into out_buffer - static in order */
 void
 exp_init_log()
 {
-	out_buffer = malloc(BUFSIZ);
+	out_buffer = ckalloc(BUFSIZ);
 	out_buffer_size = BUFSIZ;
 	outp_last = out_buffer + BUFSIZ - 1;
 }
