@@ -7,14 +7,10 @@ dollars.  Therefore it is public domain.  However, the author and NIST
 would appreciate credit if this program or parts of it are used.
 */
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #include <stdio.h>
 #include "tcl.h"
-#include "exp_global.h"
-#include "exp_rename.h"
-#include "exp_command.h"
-#include "exp_log.h"
-#include "exp_main.h"
+#include "expect_tcl.h"
 
 void
 main(argc, argv)
@@ -23,7 +19,16 @@ char *argv[];
 {
 	Tcl_Interp *interp = Tcl_CreateInterp();
 
-	exp_init(interp);
+	if (Tcl_Init(interp) == TCL_ERROR) {
+		fprintf(stderr,"Tcl_Init failed: %s\n",interp->result);
+		exit(1);
+	}
+
+	if (Exp_Init(interp) == TCL_ERROR) {
+		fprintf(stderr,"Exp_Init failed: %s\n",interp->result);
+		exit(1);
+	}
+
 	exp_parse_argv(interp,argc,argv);
 
 	/* become interactive if requested or "nothing to do" */
