@@ -1,32 +1,36 @@
 /* exp_main_sub.c - miscellaneous subroutines for Expect or Tk main() */
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #include <stdio.h>
+#include <errno.h>
 #include "tcl.h"
 #include "tclInt.h"
 #include "exp_rename.h"
-#include "exp_main.h"
-#include "exp_global.h"
+#include "exp_prog.h"
 #include "exp_command.h"
-#include "exp_tty.h"
+#include "exp_tty_in.h"
 #include "exp_log.h"
 #include "exp_event.h"
-#include "exp_main.h"
 #ifdef TCL_DEBUGGER
 #include "Dbg.h"
 #endif
 
-#ifdef __SABER__
+#ifdef __CENTERLINE__
 #undef	VERSION
-#define	VERSION		"Saber"
+#define	VERSION		"5.0.3"		/* I give up! */
+					/* It is not necessary that number */
+					/* be accurate.  It is just here to */
+					/* pacify Centerline which doesn't */
+					/* seem to be able to get it from */
+					/* the Makefile. */
 #undef	SCRIPTDIR
-#define SCRIPTDIR	"test/"
+#define SCRIPTDIR	"example/"
+#undef	EXECSCRIPTDIR
+#define EXECSCRIPTDIR	"example/"
 #endif
 static char exp_version[] = VERSION;
-#define VERSION_VARNAME   "expect_version"
-#define NEED_TCL_MAJOR		6
-#define NEED_TCL_MINOR		5
-#define SCRIPTDIR_VARNAME "expect_library"
+#define NEED_TCL_MAJOR		7
+#define NEED_TCL_MINOR		0
 
 char *exp_argv0 = "this program";	/* default program name */
 void (*exp_app_exit)() = 0;
@@ -35,20 +39,21 @@ FILE *exp_cmdfile = 0;
 char *exp_cmdfilename = 0;
 int exp_cmdlinecmds = FALSE;
 int exp_interactive =  FALSE;
+int exp_buffer_command_input = FALSE;
 int exp_fgets();
 
 Tcl_Interp *exp_interp;	/* for use by signal handlers who can't figure out */
 			/* the interpreter directly */
 int exp_tcl_debugger_available = FALSE;
 
-int exp_pid;
+int exp_getpid;
 
 static void
 usage(interp)
 Tcl_Interp *interp;
 {
 	errorlog("usage: expect [-di] [-c cmds] [[-f] cmdfile] [args]\r\n");
-	exp_exit(interp,-1);
+	exp_exit(interp,1);
 }
 
 void
@@ -56,20 +61,8 @@ exp_exit(interp,status)
 Tcl_Interp *interp;
 int status;
 {
-	extern int exp_forked;
-
 	/* use following checks to prevent recursion in exit handlers */
-	static int called_app_exit = FALSE;
-	static int called_event_exit = FALSE;
-	static int called_generic_sighandler = FALSE;
-
-	if (!called_generic_sighandler) {
-		called_generic_sighandler = TRUE;
-	/* called user-defined exit routine if one exists */
-	exp_generic_sighandler(0);
-	} else {
-		debuglog("onexit handler called recursively - forcing exit\r\n");
-	}
+	static int did_event_exit = FALSE;
 
 	if (!interp) {
 		/* if no interp handy (i.e., called from interrupt handler) */
@@ -78,26 +71,64 @@ int status;
 		interp = exp_interp;
 	}
 
-	if (exp_app_exit) {
-		if (!called_app_exit) {
-			called_app_exit = TRUE;
-			(*exp_app_exit)(interp);
-		} else {
-			debuglog("application exit handler called recursively - forcing exit\r\n");
-		}
-	}
+	exp_exit_handlers(interp);
 
 	if (exp_event_exit) {
-		if (!called_event_exit) {
-			called_event_exit = TRUE;
+		if (!did_event_exit) {
+			did_event_exit = TRUE;
 			(*exp_event_exit)(interp);
 		} else {
 			debuglog("event exit handler called recursively - forcing exit\r\n");
 		}
 	}
 
-	if (!exp_disconnected && !exp_forked
-	    && (exp_dev_tty != -1) && isatty(exp_dev_tty) && exp_ioctled_devtty) {
+	Tcl_DeleteInterp(interp);
+
+	exit(status);
+}
+
+void
+exp_exit_handlers(interp)
+Tcl_Interp *interp;
+{
+	extern int exp_forked;
+
+	/* use following checks to prevent recursion in exit handlers */
+	static int did_app_exit = FALSE;
+	static int did_expect_exit = FALSE;
+
+	if (!interp) {
+		/* if no interp handy (i.e., called from interrupt handler) */
+		/* use last one created - it's a hack but we're exiting */
+		/* ungracefully to begin with */
+		interp = exp_interp;
+	}
+
+	if (!did_expect_exit) {
+		did_expect_exit = TRUE;
+		/* called user-defined exit routine if one exists */
+		if (exp_onexit_action) {
+			int result = Tcl_GlobalEval(interp,exp_onexit_action);
+			if (result != TCL_OK) exp_background_error(interp);
+		}
+	} else {
+		debuglog("onexit handler called recursively - forcing exit\r\n");
+	}
+
+	if (exp_app_exit) {
+		if (!did_app_exit) {
+			did_app_exit = TRUE;
+			(*exp_app_exit)(interp);
+		} else {
+			debuglog("application exit handler called recursively - forcing exit\r\n");
+		}
+	}
+
+	if (!exp_disconnected
+	    && !exp_forked
+	    && (exp_dev_tty != -1)
+	    && isatty(exp_dev_tty)
+	    && exp_ioctled_devtty) {
 		exp_tty_set(interp,&exp_tty_original,exp_dev_tty,0);
 	}
 	/* all other files either don't need to be flushed or will be
@@ -107,12 +138,9 @@ int status;
 	   which will blow them away if not prepared.
 	*/
 
-	Tcl_DeleteInterp(interp);
-
 #ifdef CRAY
 	ttyp_reset();
 #endif
-	exit(status);
 }
 
 
@@ -125,7 +153,7 @@ static char prompt1_default[] = "expect%d.%d> ";
 
 /*ARGSUSED*/
 int
-cmdPrompt1(clientData, interp, argc, argv)
+Exp_Prompt1Cmd(clientData, interp, argc, argv)
 ClientData clientData;
 Tcl_Interp *interp;
 int argc;
@@ -140,7 +168,7 @@ char **argv;
 
 /*ARGSUSED*/
 int
-cmdPrompt2(clientData, interp, argc, argv)
+Exp_Prompt2Cmd(clientData, interp, argc, argv)
 ClientData clientData;
 Tcl_Interp *interp;
 int argc;
@@ -168,13 +196,38 @@ char *s;		/* function name */
 	       );
 }
 
+/* handle an error from Tcl_Eval or Tcl_EvalFile */
+static void
+handle_eval_error(interp,check_for_nostack)
+Tcl_Interp *interp;
+int check_for_nostack;
+{
+	char *msg;
+
+	/* if errorInfo has something, print it */
+	/* else use what's in interp->result */
+
+	msg = Tcl_GetVar(interp,"errorInfo",TCL_GLOBAL_ONLY);
+	if (!msg) msg = interp->result;
+	else if (check_for_nostack) {
+		/* suppress errorInfo if generated via */
+		/* error ... -nostack */
+		if (0 == strncmp("-nostack",msg,8)) return;
+	}
+
+	/* no \n at end, since ccmd will already have one. */
+	/* Actually, this is not true if command is last in */
+	/* file and has no newline after it, oh well */
+	errorlog("%s\r\n",exp_cook(msg,(int *)0));
+}
+
 /* user has pressed escape char from interact or somehow requested expect.
 If a user-supplied command returns:
 
 TCL_ERROR,	assume user is experimenting and reprompt
 TCL_OK,		ditto
 TCL_RETURN,	return TCL_OK (assume user just wants to escape() to return)
-TCL_RETURN_TCL,	return TCL_RETURN
+EXP_TCL_RETURN,	return TCL_RETURN
 anything else	return it
 */
 int
@@ -183,9 +236,9 @@ Tcl_Interp *interp;
 {
 	int rc;
 	char *ccmd;		/* pointer to complete command */
-	char line[BUFSIZ];	/* space for partial command */
+	char line[BUFSIZ+1];	/* space for partial command */
 	int newcmd = TRUE;
-	Tcl_CmdBuf buffer;
+	Tcl_DString dstring;
 	Interp *iPtr = (Interp *)interp;
 	int tty_changed = FALSE;
 
@@ -194,37 +247,25 @@ Tcl_Interp *interp;
 
 	int dummy;
 	int fd = fileno(stdin);
-
+	
 	expect_key++;
 
-	if (!(buffer = Tcl_CreateCmdBuf())) {
-		/* why are we doing this, this way rather than, say, */
-		/* AppendResult? */
-		strcpy(interp->result,"no more space for cmd buffer\r\n");
-		return(TCL_ERROR);
-	}
+	Tcl_DStringInit(&dstring);
+
 	newcmd = TRUE;
 	while (TRUE) {
 		/* force terminal state */
 		tty_changed = exp_tty_cooked_echo(interp,&tty_old,&was_raw,&was_echo);
 
 		if (newcmd) {
-#if TCL_MAJOR_VERSION == 6
-			rc = Tcl_Eval(interp,prompt1,0,(char **)0);
-#else
 			rc = Tcl_Eval(interp,prompt1);
-#endif
-			if (rc == TCL_OK) Log(1,"%s",interp->result);
-			else Log(1,prompt1_default,iPtr->numLevels,
+			if (rc == TCL_OK) exp_log(1,"%s",interp->result);
+			else exp_log(1,prompt1_default,iPtr->numLevels,
 						   iPtr->curEventNum+1);
 		} else {
-#if TCL_MAJOR_VERSION == 6
-			rc = Tcl_Eval(interp,prompt2,0,(char **)0);
-#else
 			rc = Tcl_Eval(interp,prompt2);
-#endif
-			if (rc == TCL_OK) Log(1,"%s",interp->result);
-			else Log(1,prompt2_default,1);
+			if (rc == TCL_OK) exp_log(1,"%s",interp->result);
+			else exp_log(1,prompt2_default,1);
 		}
 
 #ifdef SHARE_CMD_BUFFER
@@ -233,27 +274,35 @@ Tcl_Interp *interp;
 			else exp_exit(interp,0);
 		}
 #else
-
-		/* following lines which test for input aren't necessary for */
-		/* non-Tk apps, but don't hurt */
-		fs[fd].force_read = 1;
+		exp_fs[fd].force_read = 1;
 		rc = exp_get_next_event(interp,&fd,1,&dummy,EXP_TIME_INFINITY,
-			fs[fd].key);
+			exp_fs[fd].key);
 		/*  check for rc == EXP_TCLERROR? */
-		
-		
+
+		if (rc != EXP_EOF) {
+			rc = read(0,line,BUFSIZ);
+#ifdef SIMPLE_EVENT
+			if (rc == -1 && errno == EINTR) {
+				if (tcl_AsyncReady) {
+					(void) Tcl_AsyncInvoke(interp,TCL_OK);
+				}
+				continue;
+			}
+#endif
+			if (rc <= 0) {
+				if (!newcmd) line[0] = 0;
+				else rc = EXP_EOF;
+			} else line[rc] = '\0';
+		}
+
+#ifdef BEFORE_ASYNC
 		if (rc != EXP_EOF) {
 			if (0 >= (rc = read(0,line,BUFSIZ))) {
 				if (!newcmd) line[0] = 0;
 				else rc = EXP_EOF;
 			} else line[rc] = '\0';
-#if 0
-			if (fgets(line,BUFSIZ,stdin) == NULL) {
-				if (!newcmd) line[0] = 0;
-				else rc = EXP_EOF;
-			}
-#endif
 		}
+#endif
 
 		if (rc == EXP_EOF) exp_exit(interp,0);
 
@@ -265,28 +314,31 @@ Tcl_Interp *interp;
 		/* typing it */
 #endif /*SHARE_CMD_BUFFER*/
 
-		if (NULL == (ccmd = Tcl_AssembleCmd(buffer,line))) {
+		ccmd = Tcl_DStringAppend(&dstring,line,rc);
+		if (!Tcl_CommandComplete(ccmd)) {
 			newcmd = FALSE;
 			continue;	/* continue collecting command */
 		}
 		newcmd = TRUE;
 
 		if (tty_changed) exp_tty_set(interp,&tty_old,was_raw,was_echo);
-		switch (rc = Tcl_RecordAndEval(interp,ccmd,0)) {
+		rc = Tcl_RecordAndEval(interp,ccmd,0);
+		Tcl_DStringFree(&dstring);
+		switch (rc) {
 		case TCL_OK:
 			if (*interp->result != 0)
-				Log(1,"%s\r\n",exp_cook(interp->result,(int *)0));
+				exp_log(1,"%s\r\n",exp_cook(interp->result,(int *)0));
 			continue;
 		case TCL_ERROR:
-			errorlog("%s\r\n",exp_cook(get_var("errorInfo"),(int *)0));
-			/* since user is typing by hand, we expect lots
-			   of errors, and want to give another chance */
+			handle_eval_error(interp,1);
+			/* since user is typing by hand, we expect lots */
+			/* of errors, and want to give another chance */
 			continue;
 #define finish(x)	{rc = x; goto done;}
 		case TCL_BREAK:
 		case TCL_CONTINUE:
 			finish(rc);
-		case TCL_RETURN_TCL:
+		case EXP_TCL_RETURN:
 			finish(TCL_RETURN);
 		case TCL_RETURN:
 			finish(TCL_OK);
@@ -300,15 +352,14 @@ Tcl_Interp *interp;
  done:
 	if (tty_changed) exp_tty_set(interp,&tty_old,was_raw,was_echo);
 
-	/* currently, code guarantees buffer is valid */
-	Tcl_DeleteCmdBuf(buffer);
+	Tcl_DStringFree(&dstring);
 
 	return(rc);
 }
 
 /*ARGSUSED*/
 int
-cmdExpectVersion(clientData, interp, argc, argv)
+Exp_ExpVersionCmd(clientData, interp, argc, argv)
 ClientData clientData;
 Tcl_Interp *interp;
 int argc;
@@ -331,19 +382,20 @@ char **argv;
 	umajor = atoi(user_version);
 
 	/* first check major numbers */
-	if (emajor > umajor) return(TCL_OK);
-	else if (emajor == umajor) {
+	if (emajor == umajor) {
 		int u, e;
 
 		/* now check minor numbers */
 		char *dot = strchr(user_version,'.');
-		/* if user did not supply minor number, let it go */
-		if (dot) {
-			u = atoi(dot+1);
-			dot = strchr(exp_version,'.');
-			e = atoi(dot+1);
-			if (e >= u) return(TCL_OK);
+		if (!dot) {
+			exp_error(interp,"version number must include a minor version number");
+			return TCL_ERROR;
 		}
+
+		u = atoi(dot+1);
+		dot = strchr(exp_version,'.');
+		e = atoi(dot+1);
+		if (e >= u) return(TCL_OK);
 	}
 
 	if (argc == 2) {
@@ -353,12 +405,14 @@ char **argv;
 	}
 	errorlog("%s: requires Expect version %s (but using %s)\r\n",
 		exp_argv0,user_version,exp_version);
-	exp_exit(interp,-1);
+	exp_exit(interp,1);
 	/*NOTREACHED*/
 }
 
-void
-exp_init(interp)
+static char init_auto_path[] = "lappend auto_path $exp_library $exp_exec_library";
+
+int
+Exp_Init(interp)
 Tcl_Interp *interp;
 {
 	static int first_time = TRUE;
@@ -368,15 +422,16 @@ Tcl_Interp *interp;
 		char *dot = strchr(TCL_VERSION,'.');
 		int tcl_minor = atoi(dot+1);
 
-		if (tcl_major < NEED_TCL_MAJOR || tcl_minor < NEED_TCL_MINOR) {
-			fprintf(stderr,
+		if (tcl_major < NEED_TCL_MAJOR || 
+		    (tcl_major == NEED_TCL_MAJOR && tcl_minor < NEED_TCL_MINOR)) {
+			sprintf(interp->result,
 			   "%s compiled with Tcl %d.%d but needs at least Tcl %d.%d\n",
 				exp_argv0,tcl_major,tcl_minor,
 				NEED_TCL_MAJOR,NEED_TCL_MINOR);
-		       exit(-1);
+			return TCL_ERROR;
 		}
 
-		exp_pid = getpid();
+		exp_getpid = getpid();
 		exp_init_pty();
 		exp_init_tty(); /* do this only now that we have looked at */
 				/* original tty state */
@@ -384,7 +439,7 @@ Tcl_Interp *interp;
 		exp_init_event();
 		exp_init_trap();
 		exp_init_unit_random();
-		exp_init_spawn_ids(interp);
+		exp_init_spawn_ids();
 
 		first_time = FALSE;
 	}
@@ -392,35 +447,44 @@ Tcl_Interp *interp;
 	/* save last known interp for emergencies */
 	exp_interp = interp;
 
-	/* call explicitly, so user can put history info in init prompt */
-	Tcl_InitHistory(interp);
-
 	/* initialize commands */
-	exp_create_commands(interp);/* add non-expect cmds to interpreter */
-	exp_init_expect(interp);	/* add expect cmds to interpreter */
+	exp_init_most_cmds(interp);	/* add misc     cmds to interpreter */
+	exp_init_expect_cmds(interp);	/* add expect   cmds to interpreter */
+	exp_init_main_cmds(interp);	/* add main     cmds to interpreter */
+	exp_init_trap_cmds(interp);	/* add trap     cmds to interpreter */
+	exp_init_tty_cmds(interp);	/* add tty      cmds to interpreter */
+	exp_init_interact_cmds(interp);	/* add interact cmds to interpreter */
+
 	exp_init_spawn_id_vars(interp);
 
-	Tcl_SetVar(interp,SCRIPTDIR_VARNAME,SCRIPTDIR,0);
-	Tcl_SetVar(interp,VERSION_VARNAME,VERSION,0);
+	Tcl_SetVar(interp,"expect_library",SCRIPTDIR,0);/* deprecated */
+	Tcl_SetVar(interp,"exp_library",SCRIPTDIR,0);
+	Tcl_SetVar(interp,"exp_exec_library",EXECSCRIPTDIR,0);
+	Tcl_Eval(interp,init_auto_path);
 
 #ifdef TCL_DEBUGGER
 	Dbg_IgnoreFuncs(interp,ignore_procs);
-	Dbg_Interactor(interp,exp_interpreter);
+/*	Dbg_Interactor(interp,exp_interpreter);*/
 #endif
+
+	return TCL_OK;
 }
 
+static char sigexit_init_default[] = "trap exit {SIGINT SIGTERM}";
+static char debug_init_default[] = "trap {exp_debug 1} SIGINT";
+
 void
 exp_parse_argv(interp,argc,argv)
 Tcl_Interp *interp;
 int argc;
 char **argv;
 {
-/*	static char *cmdfilename = 0;*/
+	char argc_rep[10]; /* enough space for storing literal rep of argc */
 
 	int sys_rc = TRUE;	/* read system rc file */
 	int my_rc = TRUE;	/* read personal rc file */
 
-	int read_buffer = FALSE;/* read in entire cmdfile at once */
+	int exp_buffer_command_input = FALSE;/* read in entire cmdfile at once */
 
 	int c;
 	int rc;
@@ -428,21 +492,28 @@ char **argv;
 	extern int optind;
 	extern char *optarg;
 	char *args;		/* ptr to string-rep of all args */
+	char *debug_init;
 
 	exp_argv0 = argv[0];
 
+#ifdef TCL_DEBUGGER
+	Dbg_ArgcArgv(argc,argv,1);
+#endif
+
+	/* initially, we must assume we are not interactive */
+	/* this prevents interactive weirdness courtesy of unknown via -c */
+	/* after handling args, we can change our mind */
+	Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);
+
+	Tcl_Eval(interp,sigexit_init_default);
+
 	while ((c = getopt(argc, argv, "b:c:dD:f:inN")) != EOF) {
 		switch(c) {
 		case 'c': /* command */
 			exp_cmdlinecmds = TRUE;
-#if TCL_MAJOR_VERSION == 6
-			rc = Tcl_Eval(interp,optarg,0,(char **)0);
-#else
 			rc = Tcl_Eval(interp,optarg);
-#endif
-
 			if (rc != TCL_OK) {
-			    errorlog("%s\r\n",exp_cook(get_var("errorInfo"),(int *)0));
+			    errorlog("%s\r\n",exp_cook(Tcl_GetVar(interp,"errorInfo",TCL_GLOBAL_ONLY),(int *)0));
 			}
 			break;
 		case 'd': exp_is_debugging = TRUE;
@@ -454,12 +525,16 @@ char **argv;
 			if (Tcl_GetInt(interp,optarg,&rc) != TCL_OK) {
 				errorlog("%s: -D argument must be 0 or 1\r\n",
 					exp_argv0);
-				exp_exit(interp,-1);
+				exp_exit(interp,1);
 			}
-			if (rc) {
-				Dbg_On(interp,0);
+
+			/* set up trap handler before Dbg_On so user does */
+			/* not have to see it at first debugger prompt */
+			if (0 == (debug_init = getenv("EXPECT_DEBUG_INIT"))) {
+				debug_init = debug_init_default;
 			}
-			Dbg_ArgcArgv(argc,argv,1);
+			Tcl_Eval(interp,debug_init);
+			if (rc == 1) Dbg_On(interp,0);
 			break;
 #endif
 		case 'f': /* name of cmd file */
@@ -467,7 +542,7 @@ char **argv;
 			break;
 		case 'b': /* read cmdfile one part at a time */
 			exp_cmdfilename = optarg;
-			read_buffer = TRUE;
+			exp_buffer_command_input = TRUE;
 			break;
 		case 'i': /* interactive */
 			exp_interactive = TRUE;
@@ -490,50 +565,58 @@ char **argv;
 	/* if user hasn't explicitly requested we be interactive */
 	/* look for a file or some other source of commands */
 	if (!exp_interactive) {
-	/* get cmd file name, if we haven't got it already */
+		/* get cmd file name, if we haven't got it already */
 		if (!exp_cmdfilename && (optind < argc)) {
 			exp_cmdfilename = argv[optind];
-		optind++;
-	}
+			optind++;
+		}
 
 		if (exp_cmdfilename) {
 			if (streq(exp_cmdfilename,"-")) {
-			exp_cmdfile = stdin;
+				exp_cmdfile = stdin;
 				exp_cmdfilename = 0;
-			} else if (read_buffer) {
+			} else if (exp_buffer_command_input) {
 				exp_cmdfile = fopen(exp_cmdfilename,"r");
 				if (exp_cmdfile) {
 					exp_cmdfilename = 0;
-					exp_close_on_exec(exp_cmdfile);
+					exp_close_on_exec(fileno(exp_cmdfile));
 				} else {
-					errorlog("%s: %s\r\n",exp_cmdfilename,sys_errlist[errno]);
-			exp_exit(interp,-1);
-		}
+					errorlog("%s: %s\r\n",exp_cmdfilename,strerror(errno));
+					exp_exit(interp,1);
+				}
 			}
 		} else if (!exp_cmdlinecmds) {
-			/* no other source of commands, force interactive */
-			exp_interactive = TRUE;
+			if (isatty(0)) {
+				/* no other source of commands, force interactive */
+				exp_interactive = TRUE;
+			} else {
+				/* read cmds from redirected stdin */
+				exp_cmdfile = stdin;
+			}
 		}
 	}
 
-	/* collect remaining args and make into an argv */
-	/* Tcl expects argv[0] to be the command name, but it doesn't do */
-	/* anything useful with it anyway, so just back up the pointer */
-	/* actually, back it up twice, so we can make the resultant [0] */
-	/* be the program name */
-	/* Oh, and we add to argc for the same reason */
-	/* If no cmdfilename, there certainly can't be any args! */
-	if (exp_cmdfilename) {
-		argv[optind-1] = exp_cmdfilename;
-	} else  {
-		argv[optind-1] = exp_argv0;
+	if (exp_interactive) {
+		Tcl_SetVar(interp, "tcl_interactive","1",TCL_GLOBAL_ONLY);
 	}
 
-		Tcl_ListCmd((ClientData)0,interp,2+argc-optind,argv+optind-2);
-		args = interp->result;
+	/* collect remaining args and make into argc, argv0, and argv */
+	sprintf(argc_rep,"%d",argc-optind);
+	Tcl_SetVar(interp,"argc",argc_rep,0);
+	debuglog("set argc %s\r\n",argc_rep);
 
+	if (exp_cmdfilename) {
+		Tcl_SetVar(interp,"argv0",exp_cmdfilename,0);
+		debuglog("set argv0 \"%s\"\r\n",exp_cmdfilename);
+	} else {
+		Tcl_SetVar(interp,"argv0",exp_argv0,0);
+		debuglog("set argv0 \"%s\"\r\n",exp_argv0);
+	}
+
+	args = Tcl_Merge(argc-optind,argv+optind);
 	debuglog("set argv \"%s\"\r\n",args);
 	Tcl_SetVar(interp,"argv",args,0);
+	ckfree(args);
 
 	exp_interpret_rcfiles(interp,my_rc,sys_rc);
 }
@@ -551,7 +634,7 @@ int sys_rc;
 	    char file[200];
 	    int fd;
 
-	    sprintf(file,"%sexpect.rc",SCRIPTDIR);
+	    sprintf(file,"%s/expect.rc",SCRIPTDIR);
 	    if (-1 != (fd = open(file,0))) {
 		if (TCL_ERROR == (rc = Tcl_EvalFile(interp,file))) {
 		    errorlog("error executing system initialization file: %s\r\n",file);
@@ -559,7 +642,7 @@ int sys_rc;
 				errorlog("Tcl_Eval = %d\r\n",rc);
 		    if (*interp->result != 0)
 				errorlog("%s\r\n",interp->result);
-		    exp_exit(interp,-1);
+		    exp_exit(interp,1);
 		}
 		close(fd);
 	    }
@@ -579,7 +662,7 @@ int sys_rc;
 				errorlog("Tcl_Eval = %d\r\n",rc);
 			if (*interp->result != 0)
 				errorlog("%s\r\n",interp->result);
-			exp_exit(interp,-1);
+			exp_exit(interp,1);
 		    }
 		    close(fd);
 	        }
@@ -595,10 +678,7 @@ char *filename;
 	debuglog("executing commands from command file\r\n");
 
 	if (TCL_OK != Tcl_EvalFile(interp,filename)) {
-	    /* no \n at end, since ccmd will already have one. */
-	    /* Actually, this is not true if command is last in */
-	    /* file and has no newline after it, oh well */
-	    errorlog("%s\r\n",exp_cook(get_var("errorInfo"),(int *)0));
+		handle_eval_error(interp,0);
 	}
 }
 
@@ -610,17 +690,8 @@ FILE *fp;
 	int rc;
 	int newcmd;
 
-#if TCL_MAJOR_VERSION == 6
-	Tcl_CmdBuf buffer;
-
-	if (!(buffer = Tcl_CreateCmdBuf())) {
-		errorlog("no more space for cmd buffer\r\n");
-		exp_exit(interp,0);
-	}
-#else
 	Tcl_DString dstring;
 	Tcl_DStringInit(&dstring);
-#endif
 
 	debuglog("executing commands from command file\r\n");
 
@@ -633,36 +704,20 @@ FILE *fp;
 			if (newcmd) break;
 			else line[0] = 0;
 		}
-#if TCL_MAJOR_VERSION == 6
-		if (NULL == (ccmd = Tcl_AssembleCmd(buffer,line))) {
-#else
-		ccmd = Tcl_DStringAppend(&dstring,line,rc);
+		ccmd = Tcl_DStringAppend(&dstring,line,-1);
 		if (!Tcl_CommandComplete(ccmd)) {
-#endif
 			newcmd = FALSE;
 			continue;	/* continue collecting command */
 		}
 		newcmd = TRUE;
 
-#if TCL_MAJOR_VERSION == 6
-		rc = Tcl_Eval(interp,ccmd,0,(char **)0);
-#else
 		rc = Tcl_Eval(interp,ccmd);
-#endif
-
+		Tcl_DStringFree(&dstring);
 		if (rc != TCL_OK) {
-		    /* no \n at end, since ccmd will already have one. */
-		    /* Actually, this is not true if command is last in */
-		    /* file and has no newline after it, oh well */
-		    errorlog("%s\r\n",exp_cook(get_var("errorInfo"),(int *)0));
+			handle_eval_error(interp,0);
 		}
 	}
-#if TCL_MAJOR_VERSION == 6
-	/* currently, code guarantees buffer is valid */
-	Tcl_DeleteCmdBuf(buffer);
-#else
 	Tcl_DStringFree(&dstring);
-#endif
 }
 
 #ifdef SHARE_CMD_BUFFER
@@ -677,19 +732,19 @@ int max;
 	int write_count;/* length of first line of incoming data */
 
 	int m = fileno(stdin);
-	struct f *f;
+	struct exp_f *f;
 	int cc;
 
 	int dummy;
 
 	/* avoid returning no data, just because someone else read it in by */
 	/* passing most recent key */
-	cc = exp_get_next_event(interp,&m,1,&dummy,EXP_TIME_INFINITY,fs[m].key);
+	cc = exp_get_next_event(interp,&m,1,&dummy,EXP_TIME_INFINITY,exp_fs[m].key);
 
 	if (cc == EXP_DATA_NEW) {
 		/* try to read it */
 
-		cc = i_read(m,EXP_TIME_INFINITY);
+		cc = exp_i_read(m,EXP_TIME_INFINITY);
 
 		/* the meaning of 0 from i_read means eof.  Muck with it a */
 		/* little, so that from now on it means "no new data arrived */
@@ -697,11 +752,11 @@ int max;
 		if (cc == 0) {
 			cc = EXP_EOF;
 		} else if (cc > 0) {
-			f = fs + m;
+			f = exp_fs + m;
 			f->buffer[f->size += cc] = '\0';
 		}
 	} else if (cc == EXP_DATA_OLD) {
-		f = fs + m;
+		f = exp_fs + m;
 		cc = 0;
 	}
 
@@ -745,3 +800,17 @@ int max;
 	return(write_count);
 }
 #endif /*SHARE_CMD_BUFFER*/
+
+static struct exp_cmd_data cmd_data[]  = {
+{"expect_version",Exp_ExpVersionCmd,	0,	0},	/* deprecated */
+{"exp_version",	Exp_ExpVersionCmd,	0,	0},
+{"prompt1",	Exp_Prompt1Cmd,		0,	EXP_NOPREFIX},
+{"prompt2",	Exp_Prompt2Cmd,		0,	EXP_NOPREFIX},
+{0}};
+
+void
+exp_init_main_cmds(interp)
+Tcl_Interp *interp;
+{
+	exp_create_commands(interp,cmd_data);
+}
