@@ -16,74 +16,61 @@ Don Libes, NIST, 12/19/92
  *	can be used as the basis for main programs for other Tk
  *	applications.
  *
- * Copyright 1990-1992 Regents of the University of California.
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies.  The University of California
- * makes no representations about the suitability of this
- * software for any purpose.  It is provided "as is" without
- * express or implied warranty.
+ * Copyright (c) 1990-1993 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, without written agreement and without
+ * license or royalty fees, to use, copy, modify, and distribute this
+ * software and its documentation for any purpose, provided that the
+ * above copyright notice and the following two paragraphs appear in
+ * all copies of this software.
+ * 
+ * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
+ * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
+ * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
+ * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
+ * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
+ * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /rel/cvsfiles/devo/expect/exp_main_tk.c,v 1.12 1993/09/29 02:28:36 rob Exp $ SPRITE (Berkeley)";
+static char rcsid[] = "$Header: /rel/cvsfiles/devo/expect/exp_main_tk.c,v 1.19 1994/04/11 22:30:22 rob Exp $ SPRITE (Berkeley)";
 #endif
 
-#include "tkConfig.h"
-#include "tkInt.h"
-#ifdef TCL_DEBUGGER
+/*#include "tkConfig.h"*/
+/*#include "tkInt.h"*/
+#include "tk.h"
+#include "expect_tcl.h"
 #include "Dbg.h"
-#endif
-
-#if TCL_MAJOR_VERSION == 6
-#define tcl_eval(i,c) Tcl_Eval(i,c,0,(char **)0)
-#else
-#define tcl_eval(i,c) Tcl_Eval(i,c)
-#endif
+#include "string.h"
 
 #ifdef TK_EXTENDED
 #    include "tclExtend.h"
-Tcl_Interp *tk_mainInterp;  /* Need to process signals */
-#endif
-
-#include "exp_main.h"
-
-/*
- * Declarations for library procedures:
- */
-
-extern int isatty();
-
-/*
- * Command used to initialize wish:
- */
-
-#ifdef TK_EXTENDED
-static char initCmd[] = "load wishx.tcl";
-#else
-static char initCmd[] = "source $tk_library/wish.tcl";
 #endif
 
 /*
  * Global variables used by the main program:
  */
 
-static Tk_Window w;		/* The main window for the application.  If
+static Tk_Window mainWindow;	/* The main window for the application.  If
 				 * NULL then the application no longer
 				 * exists. */
 static Tcl_Interp *interp;	/* Interpreter for this application. */
-static int x, y;		/* Coordinates of last location moved to;
-				 * used by "moveto" and "lineto" commands. */
-#if TCL_MAJOR_VERSION == 6
-static Tcl_CmdBuf buffer;	/* Used to assemble lines of terminal input
+char *tcl_RcFileName = NULL;	/* Name of a user-specific startup script
+				 * to source if the application is being run
+				 * interactively (e.g. "~/.wishrc").  Set
+				 * by Tcl_AppInit.  NULL means don't source
+				 * anything ever. */
+static Tcl_DString command;	/* Used to assemble lines of terminal input
 				 * into Tcl commands. */
-#else
-static Tcl_DString dstring;
-#endif
 static int tty;			/* Non-zero means standard input is a
 				 * terminal-like device.  Zero means it's
 				 * a file. */
+static char errorExitCmd[] = "exit 1";
 
 /*
  * Command-line options:
@@ -99,14 +86,12 @@ char *geometry = NULL;
 int my_rc = 1;
 int sys_rc = 1;
 int optcmd_eval();
+int dashdash;	/* not used, but Tk's arg parser requires a placeholder */
 #ifdef TCL_DEBUGGER
 int optcmd_debug();
 #endif
 
 Tk_ArgvInfo argTable[] = {
-    {"-buffer", TK_ARGV_STRING, (char *) NULL, (char *) &fileName,
-	"File from which to read commands"}, /* accepted but doesn't */
-						/* actually change behavior */
     {"-file", TK_ARGV_STRING, (char *) NULL, (char *) &fileName,
 	"File from which to read commands"},
     {"-geometry", TK_ARGV_STRING, (char *) NULL, (char *) &geometry,
@@ -119,10 +104,14 @@ Tk_ArgvInfo argTable[] = {
 	"Use synchronous mode for display server"},
 
 /* for Expect */
+    {"-buffer", TK_ARGV_STRING, (char *) 1, (char *) &exp_buffer_command_input,
+	"Buffer command input"},
     {"-command", TK_ARGV_GENFUNC, (char *) optcmd_eval, (char *)0,
 	"Command(s) to execute immediately"},
-    {"-debug", TK_ARGV_CONSTANT, (char *) 1, (char *) &exp_is_debugging,
+    {"-diag", TK_ARGV_CONSTANT, (char *) 1, (char *) &exp_is_debugging,
 	"Enable diagnostics"},
+    {"--", TK_ARGV_REST, (char *)NULL, (char *)&dashdash,
+	"End of options"},
 #if TCL_DEBUGGER
     {"-Debug", TK_ARGV_GENFUNC, (char *) optcmd_debug, (char *)0, 
 	"Enable debugger"},
@@ -133,7 +122,6 @@ Tk_ArgvInfo argTable[] = {
 	"Don't read ~/.expect.rc"},
     {"-NORC", TK_ARGV_CONSTANT, (char *) 0, (char *) &sys_rc,
 	"Don't read system-wide expect.rc"},
-
     {(char *) NULL, TK_ARGV_END, (char *) NULL, (char *) NULL,
 	(char *) NULL}
 };
@@ -150,7 +138,14 @@ char **argv;
 {
 	int i;
 
+#if 0
 	exp_tcl_debugger_available = 1;
+#endif
+
+	if (argc == 0) {
+		strcpy(interp->result,"-Debug flag needs 1 or 0 argument");
+		return TCL_ERROR;
+	}
 
 	if (Tcl_GetInt(interp,argv[0],&i) != TCL_OK) {
 		return TCL_ERROR;
@@ -159,7 +154,7 @@ char **argv;
 	if (i) {
 		Dbg_On(interp,0);
 	}
-	Dbg_Interactor(interp,exp_interpreter);
+/*	Dbg_Interactor(interp,exp_interpreter);*/
 
 	argc--;
 	for (i=0;i<argc;i++) {
@@ -184,7 +179,7 @@ char **argv;
 
 	exp_cmdlinecmds = 1;
 
-	rc = tcl_eval(interp,argv[0]);
+	rc = Tcl_Eval(interp,argv[0]);
 
 	argc--;
 	for (i=0;i<argc;i++) {
@@ -199,22 +194,16 @@ char **argv;
  * procedure is only invoked if SQUARE_DEMO is defined.
  */
 
-extern int Tk_SquareCmd _ANSI_ARGS_((ClientData clientData,
-	Tcl_Interp *interp, int argc, char **argv));
+extern int SquareCmd _ANSI_ARGS_((ClientData clientData,
+	Tcl_Interp *interp, int argc, char *argv[]));
 
 /*
  * Forward declarations for procedures defined later in this file:
  */
 
-static void		DelayedMap _ANSI_ARGS_((ClientData clientData));
-static int		LinetoCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
-static int		MovetoCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
+static void		Prompt _ANSI_ARGS_((Tcl_Interp *interp, int partial));
 static void		StdinProc _ANSI_ARGS_((ClientData clientData,
 			    int mask));
-static void		StructureProc _ANSI_ARGS_((ClientData clientData,
-			    XEvent *eventPtr));
 
 /*
  *----------------------------------------------------------------------
@@ -242,9 +231,9 @@ main(argc, argv)
 {
     char *args, *p, *msg;
     char buf[20];
-    int result;
-    Tk_3DBorder border;
+    int code;
 	extern char *exp_argv0;
+	int used_argv1_for_filename = 0;	/* added for Expect - DEL */
 
 #ifdef TK_EXTENDED
     tk_mainInterp = interp = Tcl_CreateExtendedInterp();
@@ -255,7 +244,10 @@ main(argc, argv)
     Tcl_InitMemory(interp);
 #endif
 
-	exp_init(interp);
+	if (Exp_Init(interp) == TCL_ERROR) {
+		fprintf(stderr,"Exp_Init failed: %s\n",interp->result);
+		return 1;
+	}
 	exp_argv0 = argv[0];
 
 #ifdef TCL_DEBUGGER
@@ -272,7 +264,10 @@ main(argc, argv)
 	exit(1);
     }
 
-	if (!fileName) fileName = argv[1];
+    if (!fileName) {			/* added for Expect - DEL */
+	fileName = argv[1];
+        used_argv1_for_filename = 1;
+    }
 
     if (name == NULL) {
 	if (fileName != NULL) {
@@ -293,48 +288,51 @@ main(argc, argv)
     if (fileName && !exp_interactive) {
 	if (0 == strcmp(fileName,"-")) {
 		exp_cmdfile = stdin;
-	} else if (NULL == (exp_cmdfile = fopen(fileName,"r"))) {
-		perror(fileName);
-		exp_exit(interp,-1);
+	} else if (exp_buffer_command_input) {
+		if (NULL == (exp_cmdfile = fopen(fileName,"r"))) {
+			perror(fileName);
+			exp_exit(interp,1);
+		} else {
+			exp_close_on_exec(fileno(exp_cmdfile));
+		}
 	} else {
-		exp_close_on_exec(exp_cmdfile);
+		exp_cmdfilename = fileName;
 	}
     } else if (!exp_cmdlinecmds) {
 	/* no other source of commands, force interactive */
 	exp_interactive = 1;
     }
 
-    exp_interpret_rcfiles(interp,my_rc,sys_rc);
-
     /*
-     * Initialize the Tk application and arrange to map the main window
-     * after the startup script has been executed, if any.  This way
-     * the script can withdraw the window so it isn't ever mapped
-     * at all.
+     * If a display was specified, put it into the DISPLAY
+     * environment variable so that it will be available for
+     * any sub-processes created by us.
      */
 
-    w = Tk_CreateMainWindow(interp, display, name);
-    if (w == NULL) {
+    if (display != NULL) {
+	Tcl_SetVar2(interp, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);
+    }
+
+    /*
+     * Initialize the Tk application.
+     */
+
+#if TCL_MINOR_VERSION == 0
+    mainWindow = Tk_CreateMainWindow(interp, display, name);
+#else
+    mainWindow = Tk_CreateMainWindow(interp, display, name, "Tk");
+#endif
+    if (mainWindow == NULL) {
 	fprintf(stderr, "%s\n", interp->result);
 	exit(1);
     }
-    Tk_SetClass(w, "Tk");
-    Tk_CreateEventHandler(w, StructureNotifyMask, StructureProc,
-	    (ClientData) NULL);
-    Tk_DoWhenIdle(DelayedMap, (ClientData) NULL);
+#if TCL_MINOR_VERSION == 0
+    Tk_SetClass(mainWindow, "Tk");
+#endif
     if (synchronize) {
-	XSynchronize(Tk_Display(w), True);
+	XSynchronize(Tk_Display(mainWindow), True);
     }
-    Tk_GeometryRequest(w, 200, 200);
-    border = Tk_Get3DBorder(interp, w, None, "#ffe4c4");
-    if (border == NULL) {
-	Tcl_SetResult(interp, (char *) NULL, TCL_STATIC);
-	Tk_SetWindowBackground(w, WhitePixelOfScreen(Tk_Screen(w)));
-    } else {
-	Tk_SetBackgroundFromBorder(w, border);
-    }
-    XSetForeground(Tk_Display(w), DefaultGCOfScreen(Tk_Screen(w)),
-	    BlackPixelOfScreen(Tk_Screen(w)));
+    Tk_GeometryRequest(mainWindow, 200, 200);
 
     /*
      * Make command-line arguments available in the Tcl variables "argc"
@@ -342,34 +340,74 @@ main(argc, argv)
      * specified on the command line.
      */
 
+    if (used_argv1_for_filename) {	/* added for Expect - DEL */
+	argv++;
+	argc--;
+	/* if no script name, use interpreter name */
+	if (!argv[0] && !fileName) argv[0] = name;
+    }
+
     args = Tcl_Merge(argc-1, argv+1);
     Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
     ckfree(args);
     sprintf(buf, "%d", argc-1);
     Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
+    Tcl_SetVar(interp, "argv0", (fileName != NULL) ? fileName : argv[0],
+	    TCL_GLOBAL_ONLY);
     if (geometry != NULL) {
 	Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
     }
 
+    /*
+     * Set the "tcl_interactive" variable.
+     */
+
+    tty = isatty(0);
+    Tcl_SetVar(interp, "tcl_interactive",
+	    ((fileName == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);
+
     /*
      * Add a few application-specific commands to the application's
      * interpreter.
      */
 
-    Tcl_CreateCommand(interp, "lineto", LinetoCmd, (ClientData) w,
-	    (void (*)()) NULL);
-    Tcl_CreateCommand(interp, "moveto", MovetoCmd, (ClientData) w,
-	    (void (*)()) NULL);
 #ifdef SQUARE_DEMO
-    Tcl_CreateCommand(interp, "square", Tk_SquareCmd, (ClientData) w,
+    Tcl_CreateCommand(interp, "square", SquareCmd, (ClientData) mainWindow,
 	    (void (*)()) NULL);
 #endif
 
+    if (Tcl_Init(interp) == TCL_ERROR) {
+	fprintf(stderr,"Tcl_Init failed: %s\n",interp->result);
+	return 1;
+    }
+    if (Tk_Init(interp) == TCL_ERROR) {
+	fprintf(stderr,"Tk_Init failed: %s\n",interp->result);
+	return 1;
+    }
+
+    /* Call Exp_Init again because Tcl_Init resets auto_path, sigh. */
+    /* A better solution would be to execute Tcl/Tk_Init much earlier */
+    /* (before argc/argv is processed). */
+
+    if (Exp_Init(interp) == TCL_ERROR) {
+	fprintf(stderr,"Exp_Init failed: %s\n",interp->result);
+	return 1;
+    }
+
+#if 0
+    tcl_RcFileName = "~/.wishrc";
+
     /*
-     * Execute Wish's initialization script, followed by the script specified
-     * on the command line, if any.
+     * Invoke application-specific initialization.
      */
 
+    if (Tcl_AppInit(interp) != TCL_OK) {
+	fprintf(stderr, "Tcl_AppInit failed: %s\n", interp->result);
+    }
+#endif
+
+    exp_interpret_rcfiles(interp,my_rc,sys_rc);
+
 #ifdef TK_EXTENDED
      tclAppName	= "Wish";
      tclAppLongname = "Wish - Tk Shell";
@@ -381,70 +419,98 @@ main(argc, argv)
 		NULL);		   /* Standard default file */
 #endif
 
-    result = tcl_eval(interp, initCmd);
-    if (result != TCL_OK) {
-	goto error;
+    /*
+     * Set the geometry of the main window, if requested.
+     */
+
+    if (geometry != NULL) {
+	code = Tcl_VarEval(interp, "wm geometry . ", geometry, (char *) NULL);
+	if (code != TCL_OK) {
+	    fprintf(stderr, "%s\n", interp->result);
+	}
     }
 
+    /*
+     * Invoke the script specified on the command line, if any.
+     */
+
 	/* become interactive if requested or "nothing to do" */
 	if (exp_interactive) {
-		(void) tcl_eval(interp, "update");
 		(void) exp_interpreter(interp);
 	} else if (exp_cmdfile) {
 		exp_interpret_cmdfile(interp,exp_cmdfile);
-		(void) tcl_eval(interp, "update");
 		Tk_MainLoop();
 	} else if (exp_cmdfilename) {
 		exp_interpret_cmdfilename(interp,exp_cmdfilename);
-		(void) tcl_eval(interp, "update");
 		Tk_MainLoop();
 	}
 
-	exp_exit(interp,0);
+    /*
+     * Don't exit directly, but rather invoke the Tcl "exit" command.
+     * This gives the application the opportunity to redefine "exit"
+     * to do additional cleanup.
+     */
+
+    Tcl_Eval(interp, "exit");
+    exit(1);
 
 #if 0
-    tty = isatty(0);
     if (fileName != NULL) {
-	result = Tcl_VarEval(interp, "source ", fileName, (char *) NULL);
-	if (result != TCL_OK) {
+      Dbg_On(interp,0);
+	code = Tcl_VarEval(interp, "source ", fileName, (char *) NULL);
+	if (code != TCL_OK) {
 	    goto error;
 	}
 	tty = 0;
     } else {
 	/*
-	 * Commands will come from standard input.  Set up a handler
-	 * to receive those characters and print a prompt if the input
+	 * Commands will come from standard input, so set up an event
+	 * handler for standard input.  If the input device is aEvaluate the
+	 * .rc file, if one has been specified, set up an event handler
+	 * for standard input, and print a prompt if the input
 	 * device is a terminal.
 	 */
 
+	if (tcl_RcFileName != NULL) {
+	    Tcl_DString buffer;
+	    char *fullName;
+    
+	    fullName = Tcl_TildeSubst(interp, tcl_RcFileName, &buffer);
+	    if (fullName == NULL) {
+		fprintf(stderr, "%s\n", interp->result);
+	    } else {
+		if (access(fullName, R_OK) == 0) {
+		    code = Tcl_EvalFile(interp, fullName);
+		    if (code != TCL_OK) {
+			fprintf(stderr, "%s\n", interp->result);
+		    }
+		}
+	    }
+	    Tcl_DStringFree(&buffer);
+	}
 	Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
 	if (tty) {
-	    printf("wish: ");
+	    Prompt(interp, 0);
 	}
     }
     fflush(stdout);
-#if TCL_MAJOR_VERSION == 6
-    buffer = Tcl_CreateCmdBuf();
-#else
-    Tcl_DStringInit(&dstring);
-#endif
-    (void) tcl_eval(interp, "update", 0, (char **) NULL);
+    Tcl_DStringInit(&command);
 
     /*
      * Loop infinitely, waiting for commands to execute.  When there
-     * are no windows left, Tk_MainLoop returns and we clean up and
-     * exit.
+     * are no windows left, Tk_MainLoop returns and we exit.
      */
 
     Tk_MainLoop();
-    Tcl_DeleteInterp(interp);
-#if TCL_MAJOR_VERSION == 6
-    Tcl_DeleteCmdBuf(buffer);
-#else
-    Tcl_DStringFree(&dstring);
-#endif
-    exit(0);
-#endif
+
+    /*
+     * Don't exit directly, but rather invoke the Tcl "exit" command.
+     * This gives the application the opportunity to redefine "exit"
+     * to do additional cleanup.
+     */
+
+    Tcl_Eval(interp, "exit");
+    exit(1);
 
 error:
     msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
@@ -452,9 +518,10 @@ error:
 	msg = interp->result;
     }
     fprintf(stderr, "%s\n", msg);
-    tcl_eval(interp, "destroy .");
-    exit(1);
-    return 0;			/* Needed only to prevent compiler warnings. */
+    Tcl_Eval(interp, errorExitCmd);
+    return 1;			/* Needed only to prevent compiler warnings. */
+
+#endif /*0*/
 }
 
 #if 0
@@ -488,172 +555,107 @@ StdinProc(clientData, mask)
     char input[BUFFER_SIZE+1];
     static int gotPartial = 0;
     char *cmd;
-    int result, count;
+    int code, count;
 
     count = read(fileno(stdin), input, BUFFER_SIZE);
     if (count <= 0) {
 	if (!gotPartial) {
 	    if (tty) {
-		tcl_eval(interp, "destroy .", 0, (char **) NULL);
-		exit(0);
+		Tcl_Eval(interp, "exit");
+		exit(1);
 	    } else {
 		Tk_DeleteFileHandler(0);
 	    }
 	    return;
 	} else {
-	    input[0] = 0;
+	    count = 0;
+	}
+    }
+    cmd = Tcl_DStringAppend(&command, input, count);
+    if (count != 0) {
+	if ((input[count-1] != '\n') && (input[count-1] != ';')) {
+	    gotPartial = 1;
+	    goto prompt;
+	}
+	if (!Tcl_CommandComplete(cmd)) {
+	    gotPartial = 1;
+	    goto prompt;
 	}
-    } else {
-	input[count] = 0;
-#if TCL_MAJOR_VERSION == 6
-    cmd = Tcl_AssembleCmd(buffer, input);
-    if (cmd == NULL) {
-#else
-    cmd = Tcl_DStringAppend(&dstring,line,rc);
-    if (!Tcl_CommandComplete(cmd)) {
-#endif
-	gotPartial = 1;
-	return;
     }
     gotPartial = 0;
-    result = Tcl_RecordAndEval(interp, cmd, 0);
+
+    /*
+     * Disable the stdin file handler while evaluating the command;
+     * otherwise if the command re-enters the event loop we might
+     * process commands from stdin before the current command is
+     * finished.  Among other things, this will trash the text of the
+     * command being evaluated.
+     */
+
+    Tk_CreateFileHandler(0, 0, StdinProc, (ClientData) 0);
+    code = Tcl_RecordAndEval(interp, cmd, 0);
+    Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
+    Tcl_DStringFree(&command);
     if (*interp->result != 0) {
-	if ((result != TCL_OK) || (tty)) {
+	if ((code != TCL_OK) || (tty)) {
 	    printf("%s\n", interp->result);
 	}
     }
+
+    /*
+     * Output a prompt.
+     */
+
+    prompt:
     if (tty) {
-	printf("wish: ");
-	fflush(stdout);
+	Prompt(interp, gotPartial);
     }
 }
-#endif
 
 /*
  *----------------------------------------------------------------------
  *
- * StructureProc --
+ * Prompt --
  *
- *	This procedure is invoked whenever a structure-related event
- *	occurs on the main window.  If the window is deleted, the
- *	procedure modifies "w" to record that fact.
+ *	Issue a prompt on standard output, or invoke a script
+ *	to issue the prompt.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Variable "w" may get set to NULL.
+ *	A prompt gets output, and a Tcl script may be evaluated
+ *	in interp.
  *
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 static void
-StructureProc(clientData, eventPtr)
-    ClientData clientData;	/* Information about window. */
-    XEvent *eventPtr;		/* Information about event. */
+Prompt(interp, partial)
+    Tcl_Interp *interp;			/* Interpreter to use for prompting. */
+    int partial;			/* Non-zero means there already
+					 * exists a partial command, so use
+					 * the secondary prompt. */
 {
-    if (eventPtr->type == DestroyNotify) {
-	w = NULL;
-    }
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * DelayedMap --
- *
- *	This procedure is invoked by the event dispatcher once the
- *	startup script has been processed.  It waits for all other
- *	pending idle handlers to be processed (so that all the
- *	geometry information will be correct), then maps the
- *	application's main window.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The main window gets mapped.
- *
- *----------------------------------------------------------------------
- */
+    char *promptCmd;
+    int code;
 
-	/* ARGSUSED */
-static void
-DelayedMap(clientData)
-    ClientData clientData;	/* Not used. */
-{
-
-    while (Tk_DoOneEvent(TK_IDLE_EVENTS) != 0) {
-	/* Empty loop body. */
+    promptCmd = Tcl_GetVar(interp,
+	partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
+    if (promptCmd == NULL) {
+	defaultPrompt:
+	if (!partial) {
+	    fputs("% ", stdout);
+	}
+    } else {
+	code = Tcl_Eval(interp, promptCmd);
+	if (code != TCL_OK) {
+	    Tcl_AddErrorInfo(interp,
+		    "\n    (script that generates prompt)");
+	    fprintf(stderr, "%s\n", interp->result);
+	    goto defaultPrompt;
+	}
     }
-    if (w == NULL) {
-	return;
-    }
-    Tk_MapWindow(w);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * MoveToCmd and LineToCmd --
- *
- *	This procedures are registered as the command procedures for
- *	"moveto" and "lineto" Tcl commands.  They provide a trivial
- *	drawing facility.  They don't really work right, in that the
- *	drawn information isn't persistent on the screen (it will go
- *	away if the window is iconified and de-iconified again).  The
- *	commands are here partly for testing and partly to illustrate
- *	how to add application-specific commands to Tk.  You probably
- *	shouldn't use these commands in any real scripts.
- *
- * Results:
- *	The procedures return standard Tcl results.
- *
- * Side effects:
- *	The screen gets modified.
- *
- *----------------------------------------------------------------------
- */
-
-	/* ARGSUSED */
-static int
-MovetoCmd(dummy, interp, argc, argv)
-    ClientData dummy;			/* Not used. */
-    Tcl_Interp *interp;			/* Current interpreter. */
-    int argc;				/* Number of arguments. */
-    char **argv;			/* Argument strings. */
-{
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" x y\"", (char *) NULL);
-	return TCL_ERROR;
-    }
-    x = strtol(argv[1], (char **) NULL, 0);
-    y = strtol(argv[2], (char **) NULL, 0);
-    return TCL_OK;
-}
-	/* ARGSUSED */
-static int
-LinetoCmd(dummy, interp, argc, argv)
-    ClientData dummy;			/* Not used. */
-    Tcl_Interp *interp;			/* Current interpreter. */
-    int argc;				/* Number of arguments. */
-    char **argv;			/* Argument strings. */
-{
-    int newX, newY;
-
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" x y\"", (char *) NULL);
-	return TCL_ERROR;
-    }
-    newX = strtol(argv[1], (char **) NULL, 0);
-    newY = strtol(argv[2], (char **) NULL, 0);
-    Tk_MakeWindowExist(w);
-    XDrawLine(Tk_Display(w), Tk_WindowId(w),
-	    DefaultGCOfScreen(Tk_Screen(w)), x, y, newX, newY);
-    x = newX;
-    y = newY;
-    return TCL_OK;
+    fflush(stdout);
 }
+#endif /*0*/
