@@ -8,7 +8,13 @@ would appreciate credit if this program or parts of it are used.
 
 */
 
-#include "stdlib.h"		/* for malloc */
+#include "expect_cf.h"
+#include "tcl.h"
+#ifdef NO_STDLIB_H
+#include "compat/stdlib.h"
+#else
+#include <stdlib.h>		/* for malloc */
+#endif
 
 /* generate printable versions of random ASCII strings.  Primarily used */
 /* by cmdExpect when -d forces it to print strings it is examining. */
@@ -26,8 +32,8 @@ char *s;
 	/* worst case is every character takes 3 to printify */
 	need = strlen(s)*3 + 1;
 	if (need > destlen) {
-		if (dest) free(dest);
-		dest = malloc(need);
+		if (dest) ckfree(dest);
+		dest = ckalloc(need);
 		destlen = need;
 	}
 
