@@ -8,7 +8,7 @@ would appreciate credit if this program or parts of it are used.
 
 */
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
@@ -205,9 +205,9 @@ void
 exp_pty_unlock()
 {
 	if (locked) {
-	(void) unlink(lock);
-	locked = FALSE;
-}
+		(void) unlink(lock);
+		locked = FALSE;
+	}
 }
 
 /* returns 1 if successfully locked, 0 otherwise */
