@@ -23,7 +23,7 @@ expensive.
 */
 
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #include <stdio.h>
 #include <errno.h>
 
@@ -33,8 +33,8 @@ expensive.
 
 #include "tk.h"
 
-#include "exp_global.h"
-#include "exp_command.h"	/* for struct f defs */
+#include "exp_prog.h"
+#include "exp_command.h"	/* for struct exp_f defs */
 #include "exp_event.h"
 
 /* Tk_DoOneEvent will call our filehandler which will set the following vars */
@@ -53,22 +53,117 @@ static int default_mask = TK_READABLE;
 #endif
 #endif
 
-#if 1
 /* reduce calls to malloc/free inside Tk_...FileHandler */
-#define Fast_Tk_DeleteFileHandler(fd) Tk_CreateFileHandler(fd,0,exp_filehandler,(ClientData)0)
-#endif /* 1 */
+/* Tk insists on having a valid proc here even though it isn't used */
+#define Fast_Tk_DeleteFileHandler(fd,dummyproc) \
+		Tk_CreateFileHandler(fd,0,dummyproc,(ClientData)0)
+
+static void
+exp_arm_background_filehandler_force(m)
+int m;
+{
+	Tk_CreateFileHandler(m,
+		TK_READABLE|TK_EXCEPTION,
+		exp_background_filehandler,
+		(ClientData)(exp_fs+m));
+
+	exp_fs[m].bg_status = armed;
+}
+
+void
+exp_arm_background_filehandler(m)
+int m;
+{
+	switch (exp_fs[m].bg_status) {
+	case unarmed:
+		exp_arm_background_filehandler_force(m);
+		break;
+	case disarm_req_while_blocked:
+		exp_fs[m].bg_status = blocked;	/* forget request */
+		break;
+	case armed:
+	case blocked:
+		/* do nothing */
+		break;
+	}
+}
+
+void
+exp_disarm_background_filehandler(m)
+int m;
+{
+	switch (exp_fs[m].bg_status) {
+	case blocked:
+		exp_fs[m].bg_status = disarm_req_while_blocked;
+		break;
+	case armed:
+		Tk_DeleteFileHandler(m);
+		break;
+	case disarm_req_while_blocked:
+	case unarmed:
+		/* do nothing */
+		break;
+	}
+}
+
+/* ignore block status and forcibly disarm handler - called from exp_close. */
+/* After exp_close returns, we will not have an opportunity to disarm */
+/* because the fd will be invalid, so we force it here. */
+void
+exp_disarm_background_filehandler_force(m)
+int m;
+{
+	switch (exp_fs[m].bg_status) {
+	case blocked:
+	case disarm_req_while_blocked:
+	case armed:
+		exp_fs[m].bg_status = unarmed;
+		Tk_DeleteFileHandler(m);
+		break;
+	case unarmed:
+		/* do nothing */
+		break;
+	}
+}
+
+/* this can only be called at the end of the bg handler in which */
+/* case we know the status is some kind of "blocked" */
+void
+exp_unblock_background_filehandler(m)
+int m;
+{
+	switch (exp_fs[m].bg_status) {
+	case blocked:
+		exp_arm_background_filehandler_force(m);
+		break;
+	case disarm_req_while_blocked:
+		exp_disarm_background_filehandler_force(m);
+		break;
+	}
+}
+
+/* this can only be called at the beginning of the bg handler in which */
+/* case we know the status must be "armed" */
+void
+exp_block_background_filehandler(m)
+int m;
+{
+	Fast_Tk_DeleteFileHandler(m,exp_background_filehandler);
+	exp_fs[m].bg_status = blocked;
+}
+
 
 static void exp_filehandler(clientData,mask)
 ClientData clientData;
 int mask;
 {
 	if (ready_fd == (int)clientData) {
-		Fast_Tk_DeleteFileHandler(ready_fd);
-		fs[ready_fd].armed = FALSE;
+		Fast_Tk_DeleteFileHandler(ready_fd,exp_filehandler);
+		exp_fs[ready_fd].fg_armed = FALSE;
 	} else {
-	ready_fd = (int)clientData;
-	ready_mask = mask;
-}
+		ready_fd = (int)clientData;
+		ready_mask = mask;
+	}
 }
 
 /*ARGSUSED*/
@@ -103,7 +198,7 @@ int count;
 int *fds;
 {
 	int i;
-	int flag = TK_WRITABLE;
+	int flag = TK_READABLE;
 
 	flag |= TK_EXCEPTION;
 #if 0
@@ -137,6 +232,11 @@ int key;
 {
 	static rr = 0;	/* round robin ptr */
 	int i;	/* index into in-array */
+#ifdef HAVE_PTYTRAP
+	struct request_info ioctl_info;
+#endif
+
+	int old_async_count = exp_async_count;
 
 	int timer_created = FALSE;
 	int timer_fired = FALSE;
@@ -144,7 +244,7 @@ int key;
 
 	for (;;) {
 		int m;
-		struct f *f;
+		struct exp_f *f;
 
 		/* if anything has been touched by someone else, report that */
 		/* an event has been received */
@@ -154,7 +254,7 @@ int key;
 			if (rr >= n) rr = 0;
 
 			m = masters[rr];
-			f = fs + m;
+			f = exp_fs + m;
 
 			if (f->key != key) {
 				f->key = key;
@@ -177,22 +277,32 @@ int key;
 		}
 
 		for (;;) {
-			extern int fd_max;
 			int j;
 
 			/* make sure that all fds that should be armed are */
 			for (j=0;j<n;j++) {
 				int k = masters[j];
 
-				if (!fs[k].armed) {
+				if (!exp_fs[k].fg_armed) {
 					Tk_CreateFileHandler(k,default_mask,
 							     exp_filehandler,
 							     (ClientData)k);
-					fs[k].armed = TRUE;
+					exp_fs[k].fg_armed = TRUE;
 				}
 			}
 
 			Tk_DoOneEvent(0);	/* do any event */
+
+			if (exp_async_count != old_async_count) {
+				for (i=0;i<n;i++) {
+					if (!exp_fs[masters[i]].valid) {
+						exp_error(interp,"spawn id %d has become invalid due to an asynchronous event\n",masters[i]);
+						return TCL_ERROR;
+					}
+				}
+				old_async_count = exp_async_count;
+			}
+
 			if (timer_fired) return(EXP_TIMEOUT);
 			if (ready_fd == EXP_SPAWN_ID_BAD) continue;
 
@@ -202,8 +312,8 @@ int key;
 				if (ready_fd == masters[j]) goto found;
 			}
 			/* not found */
-			Fast_Tk_DeleteFileHandler(ready_fd);
-			fs[ready_fd].armed = FALSE;
+			Fast_Tk_DeleteFileHandler(ready_fd,exp_filehandler);
+			exp_fs[ready_fd].fg_armed = FALSE;
 			ready_fd = EXP_SPAWN_ID_BAD;
 			continue;
 		found:
@@ -212,34 +322,76 @@ int key;
 
 			if (timer_created) Tk_DeleteTimerHandler(timetoken);
 
-/*#ifdef HAVE_PTYTRAP*/
-			if (ready_mask & TK_EXCEPTION) {
-#ifndef HAVE_PTYTRAP
-				return(EXP_EOF);
-#else
-				struct request_info ioctl_info;
-				if (ioctl(*master_out,TIOCREQCHECK,&ioctl_info) < 0) {
-					exp_debuglog("ioctl error on TIOCREQCHECK: %d", errno);
-					return(TCL_ERROR);
-				}
-				if (ioctl_info.request == TIOCCLOSE) {
-					return(EXP_EOF);
-				}
-				if (ioctl(*master_out, TIOCREQSET, &ioctl_info) < 0) {
-					exp_debuglog("ioctl error on TIOCREQSET after ioctl or open on slave: %d", errno);
-					}
-				/* presumably, we trapped an open here */
-				continue;
-#endif
+			/* this test should be redundant but SunOS */
+			/* raises both READABLE and EXCEPTION (for no */
+			/* apparent reason) when selecting on a plain file */
+			if (ready_mask & TK_READABLE) {
+				return EXP_DATA_NEW;
 			}
-/*#endif *//*HAVE_PTYTRAP*/
-			return(EXP_DATA_NEW);
+
+			/* ready_mask must contain TK_EXCEPTION */
+#ifndef HAVE_PTYTRAP
+			return(EXP_EOF);
+#else
+			if (ioctl(*master_out,TIOCREQCHECK,&ioctl_info) < 0) {
+				exp_debuglog("ioctl error on TIOCREQCHECK: %s", Tcl_PosixError(interp));
+				return(EXP_TCLERROR);
+			}
+			if (ioctl_info.request == TIOCCLOSE) {
+				return(EXP_EOF);
+			}
+			if (ioctl(*master_out, TIOCREQSET, &ioctl_info) < 0) {
+				exp_debuglog("ioctl error on TIOCREQSET after ioctl or open on slave: %s", strerror(errno));
+			}
+			/* presumably, we trapped an open here */
+			continue;
+#endif /* !HAVE_PTYTRAP */
 		}
 	}
 }
 
-void
-exp_usleep(usec)
+/* Having been told there was an event for a specific fd, get it */
+/* returns status, one of EOF, TIMEOUT, ERROR or DATA */
+/*ARGSUSED*/
+int
+exp_get_next_event_info(interp,fd,ready_mask)
+Tcl_Interp *interp;
+int fd;
+int ready_mask;
+{
+#ifdef HAVE_PTYTRAP
+	struct request_info ioctl_info;
+#endif
+
+	if (ready_mask & TK_READABLE) return EXP_DATA_NEW;
+
+	/* ready_mask must contain TK_EXCEPTION */
+
+#ifndef HAVE_PTYTRAP
+	return(EXP_EOF);
+#else
+	if (ioctl(fd,TIOCREQCHECK,&ioctl_info) < 0) {
+		exp_debuglog("ioctl error on TIOCREQCHECK: %s",
+				Tcl_PosixError(interp));
+		return(EXP_TCLERROR);
+	}
+	if (ioctl_info.request == TIOCCLOSE) {
+		return(EXP_EOF);
+	}
+	if (ioctl(fd, TIOCREQSET, &ioctl_info) < 0) {
+		exp_debuglog("ioctl error on TIOCREQSET after ioctl or open on slave: %s", strerror(errno));
+	}
+	/* presumably, we trapped an open here */
+	/* call it an error for lack of anything more descriptive */
+	/* it will be thrown away by caller anyway */
+	return EXP_TCLERROR;
+#endif
+}
+
+/*ARGSUSED*/
+int	/* returns TCL_XXX */
+exp_usleep(interp,usec)
+Tcl_Interp *interp;
 long usec;
 {
 	int timer_fired = FALSE;
@@ -248,25 +400,23 @@ long usec;
 
 	while (1) {
 		Tk_DoOneEvent(0);
-		if (timer_fired) return;
+		if (timer_fired) return TCL_OK;
 
 		if (ready_fd == EXP_SPAWN_ID_BAD) continue;
-		
-		Fast_Tk_DeleteFileHandler(ready_fd);
-		fs[ready_fd].armed = FALSE;
+
+		Fast_Tk_DeleteFileHandler(ready_fd,exp_filehandler);
+		exp_fs[ready_fd].fg_armed = FALSE;
 		ready_fd = EXP_SPAWN_ID_BAD;
 	}
 }
 
+static char destroy_cmd[] = "destroy .";
+
 static void
 exp_event_exit_real(interp)
 Tcl_Interp *interp;
 {
-#if TCL_MAJOR_VERSION
-	Tcl_Eval(interp,"destroy .",0,(char **)NULL);
-#else
-	Tcl_Eval(interp,"destroy .");
-#endif
+	Tcl_Eval(interp,destroy_cmd);
 }
 
 /* set things up for later calls to event handler */
@@ -276,3 +426,9 @@ exp_init_event()
 	exp_event_exit = exp_event_exit_real;
 }
 
+void
+exp_background_error(interp)
+Tcl_Interp *interp;
+{
+	Tk_BackgroundError(interp);
+}
