@@ -12,8 +12,14 @@ would appreciate credit if this program or parts of it are used.
 
 */
 
-#include "exp_conf.h"
+#include "expect_cf.h"
 #include <stdio.h>
+#include <signal.h>
+
+#if defined(SIGCLD) && !defined(SIGCHLD)
+#define SIGCHLD SIGCLD
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -74,6 +80,7 @@ static int	realgid;
 static int 	*ptys;
 static char myname[32];
 static char hostname[MAXHOSTNAMELEN];
+char *exp_pty_slave_name;
 
 static void
 pty_stty(s,name)
@@ -82,15 +89,29 @@ char *name;		/* name of pty */
 {
 #define MAX_ARGLIST 10240
 	char buf[MAX_ARGLIST];	/* overkill is easier */
+	RETSIGTYPE (*old)();	/* save old sigalarm handler */
 
-	sprintf(buf,"stty %s < %s > %s",s,name,name);
+#ifdef STTY_READS_STDOUT
+	sprintf(buf,"/bin/stty %s > %s",s,name);
+#else
+	sprintf(buf,"/bin/stty %s < %s",s,name);
+#endif
+	old = signal(SIGCHLD, SIG_DFL);
 	system(buf);
+	signal(SIGCHLD, old);	/* restore signal handler */
 }
 
-struct	termio exp_tty_original;
-
 int exp_dev_tty;	/* file descriptor to /dev/tty or -1 if none */
-int knew_dev_tty;	/* true if we had our hands on /dev/tty at any time */
+static int knew_dev_tty;/* true if we had our hands on /dev/tty at any time */
+
+#ifdef TIOCGWINSZ
+static struct winsize winsize = {0, 0};
+#endif
+#if defined(TIOCGSIZE) && !defined(TIOCGWINSZ)
+static struct ttysize winsize = {0, 0};
+#endif
+
+struct	termio exp_tty_original;
 
 #define GET_TTYTYPE	0
 #define SET_TTYTYPE	1
@@ -108,9 +129,21 @@ char *s;	/* stty args, used only if request == SET_TTYTYPE */
 			knew_dev_tty = FALSE;
 			exp_dev_tty = -1;
 		}
+#ifdef TIOCGWINSZ
+		ioctl(fd,TIOCGWINSZ,&winsize);
+#endif
+#if defined(TIOCGSIZE) && !defined(TIOCGWINSZ)
+		ioctl(fd,TIOCGSIZE,&winsize);
+#endif
 	} else {	/* type == SET_TTYTYPE */
 		if (ttycopy && knew_dev_tty) {
 			(void) ioctl(fd, TCSETA, (char *)&exp_tty_original);
+#ifdef TIOCSWINSZ
+			ioctl(fd,TIOCSWINSZ,&winsize);
+#endif
+#if defined(TIOCSSIZE) && !defined(TIOCSWINSZ)
+			ioctl(fd,TIOCGSIZE,&winsize);
+#endif
 		}
 
 		if (ttyinit) {
@@ -128,7 +161,7 @@ char *s;	/* stty args, used only if request == SET_TTYTYPE */
 }
 
 void
-init_pty()
+exp_init_pty()
 {
 	int npty;
 	char *myline;
@@ -142,7 +175,7 @@ init_pty()
 
 	ptys = (int *) malloc(sizeof(int)*(highpty+1));
 	if (ptys == NULL) {
-		fprintf(stderr,"init_pty:  couldn't allocate pty array\n");
+		fprintf(stderr,"exp_init_pty:  couldn't allocate pty array\n");
 		exit(1);
 	}
 	for (npty = lowpty;npty <= highpty;npty++)
@@ -230,6 +263,7 @@ getptymaster()
 		}
 		debuglog("getptymaster:  allocated %s\n",linet);
 		ptys[npty] = -1;
+		exp_pty_slave_name = linet;
 		return(master);
 	}
 	if (seteuid(realuid) == -1) {		/* back to who we are! */
@@ -238,6 +272,15 @@ getptymaster()
 	return(-1);
 }
 
+/* see comment in pty_termios.c */
+/*ARGSUSED*/
+void
+exp_slave_control(master,control)
+int master;
+int control;
+{
+}
+
 int
 getptyslave(ttycopy,ttyinit,stty_args)
 int ttycopy;
@@ -258,7 +301,14 @@ char *stty_args;
 		 return(slave);
 	}
 
-	fcntl(0,F_DUPFD,1);	/* duplicate 0 onto 1 to prepare for stty */
+	if (0 == slave) {
+		/* if opened in a new process, slave will be 0 (and */
+		/* ultimately, 1 and 2 as well) */
+
+		/* duplicate 0 onto 1 to prepare for stty */
+		fcntl(0,F_DUPFD,1);
+	}
+
 	ttytype(SET_TTYTYPE,slave,ttycopy,ttyinit,stty_args);
 	return(slave);
 }
