@@ -5,14 +5,25 @@
  *	corresponding to various UNIX-related codes, such
  *	as errno and signals.
  *
- * Copyright 1991 Regents of the University of California
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that this copyright
- * notice appears in all copies.  The University of California
- * makes no representations about the suitability of this
- * software for any purpose.  It is provided "as is" without
- * express or implied warranty.
+ * Copyright (c) 1991-1993 The Regents of the University of California.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, without written agreement and without
+ * license or royalty fees, to use, copy, modify, and distribute this
+ * software and its documentation for any purpose, provided that the
+ * above copyright notice and the following two paragraphs appear in
+ * all copies of this software.
+ * 
+ * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
+ * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
+ * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
+ * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
+ * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
+ * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  */
 
 #include "tclInt.h"
@@ -261,10 +272,8 @@ Tcl_ErrnoId()
 #ifdef ENOCSI
 	case ENOCSI: return "ENOCSI";
 #endif
-#ifdef ENODATA
-#  if ECONNREFUSED != ENODATA
+#if defined(ENODATA) && (ECONNREFUSED != ENODATA)
 	case ENODATA: return "ENODATA";
-#  endif
 #endif
 #ifdef ENODEV
 	case ENODEV: return "ENODEV";
@@ -299,12 +308,10 @@ Tcl_ErrnoId()
 #ifdef ENOSPC
 	case ENOSPC: return "ENOSPC";
 #endif
-#ifdef ENOSR
-#  if ENAMETOOLONG != ENOSR
+#if defined(ENOSR) && (ENAMETOOLONG != ENOSR)
 	case ENOSR: return "ENOSR";
-#  endif
 #endif
-#if defined(ENOSTR) && (!defined (ENOTTY) || ENOSTR != ENOTTY)
+#if defined(ENOSTR) && (!defined(ENOTTY) || (ENOTTY != ENOSTR))
 	case ENOSTR: return "ENOSTR";
 #endif
 #ifdef ENOSYM
@@ -346,10 +353,8 @@ Tcl_ErrnoId()
 #ifdef EPERM
 	case EPERM: return "EPERM";
 #endif
-#ifdef EPFNOSUPPORT
-#  if ENOLCK != EPFNOSUPPORT
+#if defined(EPFNOSUPPORT) && (ENOLCK != EPFNOSUPPORT)
 	case EPFNOSUPPORT: return "EPFNOSUPPORT";
-#  endif
 #endif
 #ifdef EPIPE
 	case EPIPE: return "EPIPE";
@@ -426,15 +431,11 @@ Tcl_ErrnoId()
 #ifdef ESUCCESS
 	case ESUCCESS: return "ESUCCESS";
 #endif
-#ifdef ETIME
-#  if ELOOP != ETIME
+#if defined(ETIME) && (ELOOP != ETIME)
 	case ETIME: return "ETIME";
-#  endif
 #endif
-#ifdef ETIMEDOUT
-#  if ENOSTR != ETIMEDOUT
+#if defined(ETIMEDOUT) && (ENOSTR != ETIMEDOUT)
 	case ETIMEDOUT: return "ETIMEDOUT";
-#  endif
 #endif
 #ifdef ETOOMANYREFS
 	case ETOOMANYREFS: return "ETOOMANYREFS";
